version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries

* Place characteristics
global provider_chars = "HospScore specialist_pc pcp_pc bed_pc hosp_np_pct"
local provider_beliefs = "high_followup_pcp low_followup_pcp cowboy_pcp comforter_pcp"
local provider_beliefs = "`provider_beliefs' high_followup_card low_followup_card cowboy_card"
global provider_beliefs = "`provider_beliefs' comforter_card"

* Patient characteristics
global demographics = "age black female"
global ses = "medianHHincome hsgrad_pct"
global patient_health = "Lnhcccom_c_j Lnnumcon_c_j Lnchr_c_j Lniez_c_j"
global patient_preference = "have_unneeded_tests see_unneeded_card ag_patient comf_patient"

global nons_vars = "${demographics} ${ses} ${patient_health} ${provider_chars}"

local full_vars = "${provider_chars} ${provider_beliefs} ${demographics} ${ses}"
global full_vars = "`full_vars' ${patient_health} ${patient_preference}"

program main
    foreach effect in patient place {
        bivariate_reg using ../temp/`effect'_effect.dta, ///
            effect("`effect'") matrix("`effect'") varlist(full_vars)
        get_lasso_selectvars using ../temp/`effect'_beta_lasso.csv, effect("`effect'")
        post_lasso using ../temp/`effect'_effect.dta, lassovars("${`effect'_lassovars}") ///
            matrix("`effect'") varlist(full_vars)
    }
    foreach effect in nons_96hrr_patient nons_96hrr_place nons_306hrr_patient nons_306hrr_place {
        get_lasso_selectvars using ../temp/`effect'_beta_lasso.csv, effect("`effect'")
        post_lasso using ../temp/`effect'_effect.dta, lassovars("${`effect'_lassovars}") ///
            matrix("`effect'") varlist(nons_vars)
    }
    patient_health_decomp
    export_table
    coefficient_dot_plots
    coefficient_dot_plots_slides
end

program bivariate_reg
    syntax using, effect(string) matrix(string) varlist(string)

    use `using', clear

    foreach var of global `varlist' {
        local var_std = "`var'_std"
        regress LnRBU_fe `var_std', robust
        matrix `matrix'_reg = nullmat(`matrix'_reg) \ (_b[`var'_std] \ _se[`var'_std])
    }
    matrix `matrix'_reg = `matrix'_reg \ (_N \ .)
end

program get_lasso_selectvars
    syntax using, effect(string)

    insheet `using', clear
    rename v1 varname
    rename v2 beta_lasso
    keep if varname != "" & varname != "(Intercept)"

    forvalues i = 1 / `=_N' {
        if (beta_lasso[`i'] != 0) {
            local `effect'_lassovars "``effect'_lassovars' `=varname[`i']'"
        }
    }
    global `effect'_lassovars = "``effect'_lassovars'"
end

program post_lasso
    syntax using, lassovars(string) matrix(string) varlist(string)

    use `using', clear
    regress LnRBU_fe `lassovars', robust
    foreach var of global `varlist' {
        capture matrix `matrix'_post_lasso = nullmat(`matrix'_post_lasso) \ ///
            (_b[`var'_std] \ _se[`var'_std])
        if (_rc == 111) matrix `matrix'_post_lasso = nullmat(`matrix'_post_lasso) \ (. \ .)
    }
    matrix `matrix'_post_lasso = `matrix'_post_lasso \ (_N \ e(r2_a))
    matrix list `matrix'_post_lasso
end

program patient_health_decomp
    use ../temp/patient_effect.dta, clear

    foreach var in ${patient_health} {
        local var_std = "`var'_std"
        regress LnRBU_fe `var_std', robust
        summarize `var'
        local sd = r(sd)
        local unstd_beta = (_b[`var'_std] / `sd')
        matrix patient_decomp = nullmat(patient_decomp) \ (_b[`var'_std], `sd', `unstd_beta')
    }
    matrix colname patient_decomp = beta sd unstd_beta
    clear
    svmat patient_decomp, names(col)
    generate measure = ""
    replace measure = "Adj. Log HCC" in 1
    replace measure = "Adj. Log Chron Cond" in 2
    replace measure = "Adj. Log Charlson" in 3
    replace measure = "Adj. Log Iezzoni" in 4
    order measure, first
    export excel using "../output/tables/patient_decomp.xlsx", ///
        sheet("bivariate_reg_health") sheetreplace firstrow(varlabels)
end

program export_table
    matrix patient_place = patient_reg, patient_post_lasso, place_reg, place_post_lasso
    matrix list patient_place
    matrix_to_txt, saving(../output/tables/PatientPlaceCovariates.txt) ///
        matrix(patient_place) format(%20.4f) title(<tab:PatientPlaceOLSPostLasso>) append

    matrix nons_patient_place = nons_306hrr_patient_post_lasso, ///
        nons_96hrr_patient_post_lasso, nons_306hrr_place_post_lasso, ///
        nons_96hrr_place_post_lasso
    matrix list nons_patient_place
    matrix_to_txt, saving(../output/tables/PatientPlaceCovariates.txt) ///
        matrix(nons_patient_place) format(%20.4f) ///
        title(<tab:NonSkinnerPatientPlaceOLSPostLasso>) append
end

program coefficient_dot_plots
    matrix Patient = patient_place[.,1..2]
    matrix Place = patient_place[.,3..4]
    foreach component in Patient Place {
        matrix colnames `component' = ols post_lasso
        clear
        svmat `component', names(col)
        drop in -2 / -1
        gen var_temp = (_N/2) + 1 - ceil(_n/2)
        gen variable = var_temp if var_temp <= 4
        replace variable = var_temp + 1 if var_temp >= 5 & var_temp <=8
        replace variable = var_temp + 2 if var_temp >= 9 & var_temp <= 10
        replace variable = var_temp + 3 if var_temp >= 11 & var_temp <= 13
        replace variable = var_temp + 4 if var_temp >= 14 & var_temp <= 21
        replace variable = var_temp + 5 if var_temp >= 22 & var_temp <= 26
        drop var_temp
        gen coefficient = mod(_n,2) == 1
        reshape wide ols post_lasso, i(variable) j(coefficient)
        rename *0 se_*
        rename *1 b_*
        gen type="`component'"
        save ../temp/`component'_coefficients, replace
    }
    use ../temp/Patient_coefficients, clear
    append using ../temp/Place_coefficients
    rename *_ols *1
    rename *_post_lasso *2
    reshape long se b, i(variable type) j(reg)
    label define regression 1 "Bivariate OLS" 2 "Post-Lasso"
    label values reg regression
    gen neg = b < 0
    gen lower = b - 1.96 * se
    gen upper = b + 1.96 * se
    generate_labels

    save ../temp/Patient_Place_coefficients.dta, replace

    * Manually generate spacing for headings on left side of graphs
    local chars_left "                                     "
    local chars_space "                                                        "
    local char_title "`chars_left' Patient Characteristics `chars_space' Place Characteristics"

    local space1 "       "
    local space2 "             "
    local space3 "         "
    local space4 "  "
    local space5 "        "
    local space6 "      "
    local measure_title "`space1' Preferences `space2' Health `space3' Census `space4'"
    local measure_title "`measure_title' Demographics `space5' Physician Preferences `space6'"
    local measure_title "`measure_title' Provider Characteristics"
    local label_lines "1 2 3 4 6 7 8 9 11 12 14 15 16 18 19 20 21 22 23 24 25 27 28 29 30 31"

    foreach type in Patient Place {
        * OLS
        use ../temp/Patient_Place_coefficients.dta if (type == "`type'" & reg == 1), clear
        twoway (dot b variable, ///
            horizontal msymbol(circle) msize(small) ///
            dcolor(none) dlcolor(none) dsymbol(none)) ///
            (rcap lower upper variable, ///
            horizontal lpattern(solid) lwidth(thin)), ///
            xline(0, lwidth(thin)) ysize(13) xsize(10) ///
            yline(`label_lines', lwidth(thin) lcolor(gs14)) ///
            ylabel(1(1)31, valuelabel angle(0) labsize(vsmall) nogrid) ///
            xlabel(#4, labsize(small)) ///
            xscale(range(-0.05(0.05)0.1)) ///
            ytitle("`char_title'", size(vsmall)) ///
            ytitle(" " "`measure_title'", suffix size(vsmall) justification(left)) ///
            title("Bivariate OLS", size(medsmall) box fcolor(gs12) lcolor(gs8) lwidth(thin) bexpand) ///
            scheme(s1mono) legend(off) saving(../temp/`type'_ols.gph)

        * Post-Lasso
        use ../temp/Patient_Place_coefficients.dta if (type == "`type'" & reg == 2), clear
        twoway (dot b variable, ///
            horizontal msymbol(circle) msize(small) mfcolor(none) ///
            dcolor(none) dlcolor(none) dsymbol(none)) ///
            (rcap lower upper variable, ///
            horizontal lpattern(solid) lwidth(thin)), ///
            xline(0, lwidth(thin)) ysize(13) xsize(10) ///
            yline(`label_lines', lwidth(thin) lcolor(gs14)) ///
            ylabel(none) ///
            xlabel(#4, labsize(small)) ///
            xscale(range(-0.05(0.05)0.1)) ///
            fxsize(47.5) ///
            ytitle("") ///
            title("Post-Lasso", size(medium) box fcolor(gs12) lcolor(gs8) lwidth(thin) bexpand) ///
            scheme(s1mono) legend(off) saving(../temp/`type'_post_lasso.gph)

        graph combine ../temp/`type'_ols.gph ../temp/`type'_post_lasso.gph, ycommon xcommon ///
            graphregion(color(white))
        graph export ../output/figures/`type'DotPlot.eps, replace
    }
end

    program generate_labels
        label define y_labels ///
            31 "Hospital Compare Score" ///
            30 "Specialists Per Capita" ///
            29 "PCP Per Capita" ///
            28 "Hospital Beds Per Capita" ///
            27 "Non-Profit Hospitals" ///
            26 " " ///
            25 "High Follow-up PCP" ///
            24 "Low Follow-up PCP" ///
            23 "Cowboy PCP" ///
            22 "Comforter PCP" ///
            21 "High Follow-up Cardiologists" ///
            20 "Low Follow-up Cardiologists" ///
            19 "Cowboy Cardiologists" ///
            18 "Comforter Cardiologists" ///
            17 " " ///
            16 "Average Age" ///
            15 "Percent Black" ///
            14 "Percent Female" ///
            13 " " ///
            12 "Median Family Income" ///
            11 "Average Education" ///
            10 " " ///
            9 "Adjusted Log HCC Score" ///
            8 "Adjusted Log Chronic Conditions" ///
            7 "Adjusted Log Charlson" ///
            6 "Adjusted Log Iezzoni" ///
            5 " " ///
            4 "Have Unneeded Tests" ///
            3 "See Unneeded Cardiologists" ///
            2 "Aggressive Patient" ///
            1 "Comforter Patient"
        label values variable y_labels
    end


program coefficient_dot_plots_slides
    use ../temp/Patient_coefficients, clear
    append using ../temp/Place_coefficients
    gen post_lasso_significant = (abs(b_post_lasso/se_post_lasso) >= 1.96 & b_post_lasso != .)
    keep variable b_ols se_ols type post_lasso_significant
    gen lower = b_ols - 1.96 * se_ols
    gen upper = b_ols + 1.96 * se_ols
    generate_labels

    save ../temp/Patient_Place_coefficients.dta, replace

    * Manually generate spacing for headings on left side of graphs
    local chars_left "    "
    local chars_space "                                                                            "
    local char_title "`chars_left' Patient Characteristics `chars_space' Place Characteristics"

    local space1 ""
    local space2 "                "
    local space3 "               "
    local space4 "         "
    local space5 "                  "
    local space6 "                            "
    local measure_title "`space1' Preferences `space2' Health `space3' Census `space4'"
    local measure_title "`measure_title' Demographics `space5' Physician Preferences `space6'"
    local measure_title "`measure_title' Provider Characteristics"
    local label_lines "1 2 3 4 6 7 8 9 11 12 14 15 16 18 19 20 21 22 23 24 25 27 28 29 30 31"

    foreach type in Patient Place {
        * OLS
        use ../temp/Patient_Place_coefficients.dta if type == "`type'", clear
        twoway (dot b variable if post_lasso_significant == 0, ///
            msymbol(circle) msize(small) ///
            dcolor(none) dlcolor(none) dsymbol(none)) ///
            (dot b variable if post_lasso_significant == 1, ///
            msymbol(circle) msize(small) mcolor(red) ///
            dcolor(none) dlcolor(none) dsymbol(none)) ///
            (rcap lower upper variable if post_lasso_significant == 0, ///
            lpattern(solid) lwidth(thin)) ///
            (rcap lower upper variable if post_lasso_significant == 1, ///
            lpattern(solid) lwidth(thin) lcolor(red)), ///
            yline(0, lwidth(thin)) ysize(10) xsize(15) ///
            xline(`label_lines', lwidth(thin) lcolor(gs14)) ///
            xlabel(1(1)31, valuelabel angle(45) labsize(vsmall) nogrid) ///
            ylabel(#4, labsize(small)) ///
            xscale(range(-0.05(0.05)0.1)) ///
            xtitle(" " "`measure_title'", size(vsmall) justification(center)) ///
            xtitle("`char_title'", suffix size(vsmall)) ///
            scheme(s1mono) legend(off)
        graph export ../output/figures/`type'DotPlotSlide.eps, replace
    }
end

* EXECUTE
main
