version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

program main
    generate_patient_data, data("../external_links/denom/denom_data")
    generate_patient_year_data, data("../external_links/denom/denom_data")
    hcc_patient_data, pct(20) startyear(1998) endyear(2008)
end

program generate_patient_data,
    syntax, data(string)

    use `data', clear
    destring zipcode, replace

    define_age
    generate_locations, id(bene_id) unit(hrrnum) unit_out(hrr)
    generate_locations, id(bene_id) unit(zipcode) unit_out(zipcode)
    generate_locations, id(bene_id) unit(state_de) unit_out(state)
    generate_locations, id(bene_id) unit(hsanum) unit_out(hsa)

    * We define movers as patients who move HRRs between the ages of 65 and 99.
    * That means they must be 98 in the year of the move, so that they are <=99 
    *in the year after the move.
    * Special care must be given to individuals who enter and leave the sample.
    gen move_count = (hrrnum != hrrnum[_n+1] & bene_id == bene_id[_n+1]) ///
        if age>=65 & age<=98 & age[_n+1]<=99
    replace move_count = 0 if move_count==.
    egen mover = max(move_count), by(bene_id)
    replace hrr_dest=. if mover==0
    replace hrr_orig=. if mover==0

    gen hmo = hmo_mo > 0 if hmo_mo!=.

    save_data ../output_local/original_data.dta, ///
        replace key(bene_id year) log("../output/data_file_manifest.log")

    **Creating a random sample of non-movers. Choosing 0.25 resembles the 5% sample (20%/4 = 5%).
    set seed 68064123
    sort bene_id year
    by bene_id: gen rand1 = runiform() if mover==0 & _n==1
    by bene_id: egen rand = min(rand1) if mover==0
    keep if mover==1 | rand<=0.25
    drop rand*

    collapse (firstnm) sdob sdod race sex hrr_orig hrr_dest zipcode_orig zipcode_dest ///
	state_orig state_dest hsa_orig hsa_dest mover (max) hmo *flag* (sum) move_count, by(bene_id)

    gen mult_mover = move_count > 1
    drop move_count
    assert zipcode_orig!=zipcode_dest if mover==1 & mult_mover==0
    assert hrr_orig!=hrr_dest if mover==1 & mult_mover==0
    assert hrr_dest == hrr_orig if mover == 0 & hrr_orig!=-999 & hrr_dest!=-999

    unique bene_id

    ** Label variables
    label var bene_id "Beneficiary ID"
    label var sdod "Date of Death"
    label var sdob "Date of Birth"
    label var race "Beneficiary Race"
    label var sex "Beneficiary Sex"
    label var zipcode_orig "First Observed Zipcode"
    label var zipcode_dest "Last Observed Zipcode"
    label var hrr_orig "First Observed HRR"
    label var hrr_dest "Last Observed HRR"
    label var mover "I: Mover Status"
    label var hmo "I: Was on HMO"
    label var mult_mover "I: Moves More Than Once"
    label var dup_bene_id_flag "I: A bene_id has a duplicate observation in any year of our sample."
    label var foreign_bene_id_flag "I: A bene_id lives outside of the US in any year of our sample."
    label var chg_bene_id_flag "I: A bene_id leaves our sample, but does not die."
    ** add value labels to race and sex
    label define race_lab 0 "unknown" 1 "white" 2 "black" 3 "other" 4 "asian" 5 "hispanic" ///
        6 "north american native"
    label define sex_lab 0 "female" 1 "male"
    label values sex sex_lab
    label values race race_lab

    save_data ../output_local/CharsPatient.dta, ///
        replace key(bene_id) log("../output/data_file_manifest.log")

    keep bene_id sdod
    save_data ../output_local/sdod_list.csv if sdod!=., outsheet delim(",") ///
        replace key(bene_id) log("../output/data_file_manifest.log")
    keep bene_id
    save_data ../output_local/samp_list.dta, ///
        replace key(bene_id) log("../output/data_file_manifest.log")
    save_data ../output_local/samp_list.csv, outsheet delim(",") ////
        replace key(bene_id) log("../output/data_file_manifest.log")

end

    program define_age
        sort bene_id year
        replace sdob = sdob[_n-1] if bene_id==bene_id[_n-1]
        gen birthyear = year(sdob)
        gen age = year-birthyear+1 if month(sdob)<= 3
        replace age = year-birthyear if month(sdob)> 3
    end

    program generate_locations
        syntax, id(varname) unit(str) unit_out(str)

        cap drop age_max age_min

        bys `id': egen age_max = max(age) if age<=99
        bys `id': egen age_min = min(age) if age>=65

        gen dest`unit' = `unit'  if `id' != `id'[_n+1] & age<=98 & age[_n+1]<=99
        replace dest`unit' = `unit' if dest`unit'==. & age==age_max
        gen orig`unit' = `unit'  if `id' != `id'[_n-1] & age>=66 & age[_n-1]<=65
        replace orig`unit' = `unit' if orig`unit'==. & age==age_min

        replace dest`unit' = -999 if dest`unit' == .
        replace orig`unit' = -999 if orig`unit' == .

        egen `unit_out'_orig = max(orig`unit'), by(`id')
        egen `unit_out'_dest = max(dest`unit'), by(`id')

        assert `unit_out'_dest!=-999 if age>=65 & age<=99
        assert `unit_out'_orig!=-999 if age>=65 & age<=99

        drop dest`unit' orig`unit'

    end

program generate_patient_year_data,
    syntax, data(string)

    use `data', clear
    merge m:1 bene_id using ../output_local/samp_list, ///
        assert(match master) keep(match)

    define_age
    generate_locations, id(bene_id) unit(hrrnum) unit_out(hrr)

    gen move_count = (hrrnum != hrrnum[_n+1] & bene_id == bene_id[_n+1]) ///
        if age>=65 & age<=98 & age[_n+1]<=99
    replace move_count = 0 if move_count==.
    egen mover = max(move_count), by(bene_id)
    replace hrr_dest=. if mover==0
    replace hrr_orig=. if mover==0
	
    gen HMO = hmo_mo > 0 if hmo_mo!=.

    ** drop entry errors (duplicate bene_id with different characteristics
    ** drop all duplicated bene_ed year entries with non matching other characteristics
    disp "DROPPING DUPLICATE OBS OF BENE_ID"
    duplicates drop bene_id year, force
	
	generate_rel_yr, id(bene_id) hrrvar(hrrnum)
	keep bene_id year age zipcode hrrnum state_de hsanum a_mo_cnt b_mo_cnt HMO rel_yr_rit

    ** Label variables
    label var bene_id "Beneficiary ID"
    label var year "Observation Year"
    label var age "Beneficiary Age"
    label var state_de "Beneficiary State"
    label var zipcode "Beneficiary Zip Code"
    label var hrrnum "Beneficiary HRR"
    label var a_mo_cnt "Months with Medicare Part A"
    label var b_mo_cnt "Months with Medicare Part B"
    label var HMO "I: Patient has HMO"

    save_data ../output_local/CharsPatientYear.dta, key(bene_id year) replace ///
        log("../output/data_file_manifest.log")

end

    program generate_rel_yr
        syntax, id(varname) hrrvar(varname)

        local id = "bene_id"
        local hrrvar = "hrrnum"

        sort `id' year
        gen moveyr = year if `hrrvar' != `hrrvar'[_n+1] & mover==1 ///
            & `id'==`id'[_n+1] & age>=65 & age<=98 & age[_n+1]<=99
		egen moveyear = min(moveyr), by(`id')
        gen rel_yr_rit = year - moveyear - 1
        drop moveyr moveyear

        assert `hrrvar'!=`hrrvar'[_n+1] if `id'==`id'[_n+1] & rel_yr_rit==-1
        assert `hrrvar'==hrr_orig if rel_yr_rit<=-1 & age>=65 & age<=98
        assert hrr_dest == hrr_orig if mover == 0 & hrr_dest!=-999 & hrr_orig!=-999

    end
	
program hcc_patient_data,
	syntax, pct(string) startyear(real) endyear(real)

    forval year = `startyear'/`endyear'{
        use if year==`year' using ../external_links/denom/denom_data, clear
        replace sex=2 if sex==0
		format sdob %td
		rename sdob dob
        keep bene_id sex dob mcaid nemcaid orec
        save_data ../output_local/hccperson_`pct'_`year'.csv, outsheet delim(",") ///
            replace key(bene_id) log("../output/data_file_manifest.log")
	}

end

* EXECUTE
main


