*********************************************************************************************;
** SAS PROGRAM TO TAKE ALL PROVIDER NUMBER WHICH APPEAR IN THE MEDPAR FILE, MERGE THEM WITH THE;
** POS FILE : GET ADDRESS INFORMATION, MAKE A WJOLE SERIES OF EDITS THEN INTERPOLATE ADDRESS INFORMATION;

** THE CODE WAS ORIGNIALLY CREATED BY MATT FIEDLER AND THEN MODIFIED BY MAURICE DALTON.
** ALL ERRORS ARE MY OWN;
*********************************************************************************************;
***********************************************************************
* DEFINE SAS OPTIONS ****;
***********************************************************************;
**READ IN SAS OPTIONS;
*Use these options for debugging;
*options symbolgen mlogic mprint mfile ;
%include "../../sasoptions.sas";

***********************************************************************;
* DEFINE DIRECTORIES****;
***********************************************************************;
%include "../../dirInit.sas";

***********************************************************************;
* READ IN SAS MACRO FOUND IN OTHER FILES  ;
***********************************************************************;
** load file containing various simple macros that make programming easier;
%include "&SASmac./UtilityMacros.sas";

***********************************************************************;
* DEFINE LIBRARIES ;
***********************************************************************;

* Use archived data results;
x "cp /disk/aging/medicare/code/ExtData/providerGeoCodingPrj/outputDTA/* /&geoex./";
x "bunzip2 /&geoex./*";

* Defines local directories depending on which server currently running;
libname geoex "&geoex.";
libname ingnis "&ingnis.";

%let writeout=&programRoot./0PreProcessPrgms/providerGeoCodingPrj/excelFilesForHandCoding;
x "mkdir &writeout. -p";
%let indir=/disk/aging/medicare/code/AuxData/providerGeoCodingPrj/orig_geocodedDta;
**** PRELIMINARIES ****;

* load file containing various simple macros that make programming easier;


* directory where extracts are read in from;
* directory where extracts should be placed;


* set libraries;
libname indir "&indir.";
libname writeout "&writeout.";



**** PART 1: READ IN GEOCODING RESULTS ****;
%macro ReadGeocodingResults(pfx, resfn);
    * read in the geocoding results;
    x "dos2unix -437 &resfn.";

    data GeoRes_&pfx.;
        infile "&resfn." delimiter="," MISSOVER DSD LRECL=32767 FIRSTOBS=2;

        informat status $1.;
        informat score best8.;
        informat match_addr $58.;
        informat X best16.;
        informat Y best16.;
        informat addrid_&pfx. 8.;

        input
            status $
            score
            match_addr $
            X
            Y
            addrid_&pfx.;

        if addrid_&pfx. ne 0;

        rename status=stat_&pfx. score=score_&pfx. X=lon_&pfx. Y=lat_&pfx. match_addr=maddr_&pfx.;
    run;

    proc freq data=GeoRes_&pfx.;
        tables stat_&pfx.;
    run;

    proc sort data=GeoRes_&pfx.;
        by addrid_&pfx.;
    run;

    %mend;

%ReadGeocodingResults(pos, &indir./POSAddresses100_1215_cat01_MEfix-Matched.csv);


**** PART 2: REMATCH ADDRESSES USING OTHER YEARS MATCHES ****;

** PART 2a: Assemble comprehensive information on the matches;
* merge this onto the geocoding results;
proc contents data=GeoRes_pos ;
run;


proc sort data=GeoRes_pos;
    by addrid_pos;
run;

data ResultList;
    merge geoex.HospAddrForExp GeoRes_pos;
    by addrid_pos;
run;


** PART 2b: Rematch records within prvnumgrp-city-state-zipcode where addresses are very similar;
* generate list of addresses that failed to geocode;
data UnmatchedList;
    set ResultList;

    if stat_pos = "U";

    keep addrid_pos prvnumgrp address city state zipcode;
run;

* generate list of addresses that successfully geocoded;
* all fields get an _d appendage to indicate that they are donor addresses;
data DonorList;
    set ResultList;
    if stat_pos ~= "U";

    rename stat_pos=stat_pos_d lat_pos=lat_pos_d lon_pos=lon_pos_d score_pos=score_pos_d maddr_pos=maddr_pos_d addrid_pos=addrid_pos_d address=address_d;
run;

* execute SQL Match;
* addresses that failed to geocode may be matched to coordinates for the same provider for another year;
* provided that the addresses are in the same city-state-zipcode and the text of the addresses is
* sufficiently similar;
proc sql;
    create table Rematches as
        select * from UnmatchedList U, DonorList D
        where (U.prvnumgrp = D.prvnumgrp) and
              (trim(U.city) = trim(D.city)) and
              (U.state = D.state) and
              (U.zipcode = D.zipcode) and
              ((U.address = "") or (compged(D.address_d, U.address) < 600));
    quit;

* take highest scoring potential donor set of coordinates;
proc print data=Rematches (obs=10);
    where prvnumgrp="040116";
run;

proc sort data=Rematches;
    by prvnumgrp city state zipcode score_pos_d;
run;

data Rematches;
    set Rematches;
    by prvnumgrp city state zipcode;
    if last.zipcode;
    keep addrid_pos stat_pos_d lat_pos_d lon_pos_d score_pos_d maddr_pos_d addrid_pos_d;
run;


** PART 2c: Combine rematched addresses with original results;
proc sort data=ResultList;
    by addrid_pos;
run;

proc sort data=Rematches;
    by addrid_pos;
run;

data ResultList;
    merge ResultList Rematches;
    by addrid_pos;
run;


**** PART 3: COMBINE THE DATASETS AND GENERATE SOME USEFUL STATISTICS ****;
** PART 3a: Merge geocoded addresses onto the universe file;
* exceute merge;
proc sort data=geoex.medparuniv100_clean;
    by addrid_pos;
run;

proc sort data=ResultList;
    by addrid_pos;
run;

data geoex.medparuniv100_partcoord;
    merge geoex.medparuniv100_clean ResultList;
    by addrid_pos;
run;

* make a working copy of the AMI universe file with some coordinates;
data WorkingData;
    set geoex.medparuniv100_partcoord;
run;


** PART 3b: Generate provider-number-level flags;
* generate desired provider-level flags;
data TempDat;
    set WorkingData;
    pFlag = stat_pos in ("M", "T");
    pdFlag = (stat_pos in ("M","T")) or (stat_pos_d in ("M","T"));
    tFlag = stat_pos = "T";
    keep prvnumgrp pFlag pdFlag tFlag;
run;

proc sort data=TempDat;
    by prvnumgrp;
run;

proc summary nway data=TempDat;
    by prvnumgrp;
    output out=ProviderFlags
        max(pFlag) = p_MatchOne
        min(pFlag) = p_MatchAll
        max(pdFlag) = pd_MatchOne
        min(pdFlag) = pd_MatchAll
        max(tFlag) = t_any / AUTONAME;
run;

** PART 3c: Generate useful tables;

* table looking at provider number/year pairings;
proc freq data=WorkingData;
    tables fileyear*stat_pos / nocol  nopercent nocum;
run;

* table look at all year match rates;
proc freq data=ProviderFlags;
    table p_MatchOne*p_MatchAll pd_MatchOne*pd_MatchAll;
run;


*** PART 4: DUMP RECORDS FOR HAND GEOCODING ***;

* Merge needed flags on;
proc sort data=WorkingData;
    by prvnumgrp fileyear;
run;

proc sort data=ProviderFlags;
    by prvnumgrp;
run;

data WorkingData;
    merge WorkingData(in=inw) ProviderFlags(in=inp);
    by prvnumgrp;
    inwork=inw;
    inProvider=inp;
run;

proc sort data=WorkingData;
    by prvnumgrp fileyear;
run;


* Dump Group A hospitals-- MATCH-NONE -- NO MATCH EXIST;
ods csv file="&writeout./GroupAHospitals.csv";
proc print data=WorkingData(where=(pd_MatchOne = 0));
    var prvnumgrp fileyear name address city state zipcode;
run;
ods csv close;


* Dump Group B hospitals -- MATCH EXIST;
ods csv file="&writeout./GroupBHospitals.csv";
proc print data=WorkingData(where=(pd_MatchOne = 1 and pd_MatchAll = 0));
    var prvnumgrp fileyear name address city state zipcode stat_pos lat_pos lon_pos;
run;
ods csv close;


* read in list of provider numbers;
filename grpCFile pipe "dos2unix < &writeout./GroupC-TOINCLUDE.csv";

data GroupCList;
    infile grpCFile delimiter="," MISSOVER DSD LRECL=32767 FIRSTOBS=2;

    informat prvnumgrp $6.;

    input
        prvnumgrp $;
run;

proc sort data=GroupCList;
    by prvnumgrp;
run;

proc sort data=WorkingData;
    by prvnumgrp fileyear;
run;

data WorkingData;
    merge WorkingData(in=inwrk) GroupCList (in=inC);
    by prvnumgrp;
    inCFlag = inC;
    if inwrk;
run;

* Dump Group C hospitals -- HAND EDITING REQUIRED;
ods csv file="&writeout./GroupCHospitals.csv";
proc print data=WorkingData(where=((pd_MatchOne = 1 and pd_MatchAll = 1 and t_any = 1) or inCFlag = 1));
    var prvnumgrp fileyear name address city state zipcode stat_pos lat_pos lon_pos;
run;
ods csv close;




