******************************************************************************;
/*******Create Cost Measure across claims files. *****************************;
******************************************************************************
*** This program aggregates the costs variables into the correct cases. It
*** then looks to create a summary at different day measures and then finally it
*** aggregates it to the index;*/

*** Note that in some case the cost are broken down by provider type, based on ;
*** the provider number in the Medpar file or the provider of service in the   ;
*** carrier files. ;

*Maurice Dalton,daltonm.at.nber.org, , based on code originally written by Matt Fiedler;

***********************************************************************;
* DEFINE SAS OPTIONS ****;
***********************************************************************;
**READ IN SAS OPTIONS;
*Use these options for debugging;
options symbolgen mlogic mprint mfile ;
%include "../../sasoptions.sas";

***********************************************************************;
* DEFINE DIRECTORIES****;
***********************************************************************;
%include "../../dirInit.sas";
%include "&SASmac./read_sysparm.sas";

* define libraries;
libname spcar "&spcar.";
libname medEx  "&medparExt.";
libname carEx   "&carExt.";
libname OpEx   "&OpExt.";
libname indxex "&indxExt.";
libname cost   "&costs.";


* load file containing various simple macros that make programming easier;
%include "&SASmac./UtilityMacros.sas";





**********************************************************************************;
***    MedPAR Cost Macro     ;
**********************************************************************************;

%macro medparStayCost;
/*DEFINE COST VARIABLES
    //The goal is to obtain an amount equal to the base DRG payment plus
    //any outlier payments.  This exlcludes:
    // 1) Indirect medical education payments;
// 2) Disproportionate share payments;
// 3) Pass-through costs (capital before 1991, direct medical education,
    //    organ acquisition costs, and certain other costs)
    // 4) Prospective payment capital payments;
//
    //For certain purposes, it may be desirable to include these amounts.  Note,
    //however, that for the latter two items, this is not possible for
    //the full 1984-2005 period.
    //
    //In particular, consistent and accurate pass-through fields are not available
    //for 1984-1986. Furthermore, capital costs were paid through prospective
    //payment for 1991-2005, but on a pass-through basis prior to 1991.  This raises
    //conceptual difficulties, plus the lack of good pass-through fields for all years
    //means that it is not possible to generate a consistent series that includes
    //these costs.
    //
    //Toward this end, the code starts with the drgprice, which is defined as described
    //at http://www.resdac.umn.edu/ddme/NewFiles/DRGPRICE.htm and
    //
    //ADDS outlier payments since we want to include these
    //
    //SUBTRACTS ime_amt for discharges after 10/1/1988 since they are added into
    //the Medicare Payment Amount variable and, thus, appear in DRGPRICE
    //(see http://www.resdac.umn.edu/ddme/NewFiles/PMT_AMT.htm)
    //
    //SUBTRACTS capital payments by subtracting off pps_cptl
    //(note that field is missing for 1984-1986, but there are no payments for
    //these years)
    //
    //SUBTRACTS disproportionate share payments by substracting off disp_shr
    //(note that field is missing for 1984-1986, but there are no payments for
    //these years)
    //
    //Pass through payments are not included in DRGPRICE and, so, do not require
    //special handling here
    */


stayCost = drgprice + outlramt ;
if passthru ne . then stayCost = stayCost +  passthru;
if dschrgdt<mdy(10,1,1988) then stayCost = stayCost + ime_amt ;
/*
if dschrgdt>=mdy(10,1,1988) then stayCost = stayCost - ime_amt ;
if pps_cptl ne . then stayCost = stayCost - pps_cptl ;
if disp_shr ne . then stayCost = stayCost - disp_shr  ;
*/


* Create separate cost variables for SNF and hospital dtays;
snf = 0;
if is_snf = 1 then snf = stayCost;

In=stayCost;
if is_snf = 1 then In = . ;

    %mend;


**********************************************************************************;
***    OutPatientCosts Macro     ;
**********************************************************************************;
%macro OutPatientCosts;

  /*
LIMITATIONS :
Prior to 4/6/93, on inpatient, outpatient, and
physician/supplier claims containing a
CLM_DISP_CD of '02', the amount shown as the Medicare
reimbursement does not take into consideration
any CWF automatic adjustments (involving erroneous
deductibles in most cases).  In as many as 30% of
the claims (30% IP, 15% OP, 5% PART B), the
reimbursement reported on the claims may be over
or under the actual Medicare payment amount.
*/


    if fileyear>=1996 then StayCost = pmt_amt + ptb_ded + ptb_coin + blddedam + prpayamt ;
    if fileyear>=1989 and fileyear <=1995 then StayCost = prvdrpmt + ptb_ded + ptb_coin + blddedam + prpayamt ;
    if fileyear >= 1986 and fileyear <= 1988 then StayCost = prvdrpmt + ptb_ded + ptb_coin + prpayamt ;

  /*//Deductible and coinsurance fields are not available for 1984 and 1985.
      //One approach would be to attempt to calculate deductible and coinsurance fields
      //from the program rules, but this would be extremely difficult.
      //
      //Rather, since deductibles are small and most services are subject to coinsurance
      //equal to 20 percent of total charges, I simply add on 20 percent of total charges.
      //On average, this is similar to the amount of cost-sharing one observes in 1986.
      //
      //It would be worthile to investigate this further at a later date.*/
          if fileyear >= 1984 & fileyear <= 1985 then StayCost = pmt_amt + .2*tot_chrg;
    %mend;



**********************************************************************************;
***    Sum costs into Variable  Macro     ;
**********************************************************************************;
%macro CreateCostVariables(days=,dNum=,pop=);

    %do i=1 %to &dNum.;
            *Flag claims which occur within XX days of index event;
    *Start summing at the day intervals defined in the macro;
        %let d=%scan(&days.,&i.);
        %let month=%syseval(&d./30);
        %put &month.;

    ** CASE 1: the claim fall completly before the day of interest. ie 7 day costs for ;
    ** a claim with los of 5 days, will simply include all costs;
        if (ClaimEnd - dgn_date)<=&d. and (ClaimEnd - dgn_date) >= 0 and stayCost ne .
            then &pop._d&d.=stayCost+&pop._d&d.;

    ** CASE 2: the length of stay on the claims falls completly after the day of interest. ie 7 day costs for ;
    ** a claim with los of 10 days would have reported zero costs. this bit of code will assume linear costs and ;
    ** the total costs occuring over 10 days and assign 70% of the costs to the 7 day cost variable. ;
        if (ClaimEnd - dgn_date)> &d. and stayCost ne .  and
            (ClaimStart - dgn_date)<= &d. then  do;
            pctDiff=(&d.+dgn_date -ClaimStart) /(ClaimEnd - ClaimStart);
            &pop._d&d.=pctDiff*stayCost+&pop._d&d.;
            end;
        %end;

    LABEL
        %do i=1 %to &dNum.;
           &pop._d&d.="Cumulative cost per patient occuring &d. days after index event at &pop."
               %end; ;

    %mend;

**********************************************************************************;
***    Outpatient process Macro     ;
**********************************************************************************;
%macro OutPatProcess(cohrt=,pct=,days=,dNum=);
    proc sort data=OpEx.&cohrt.&pct._opclms;
        by diag_id;
    run;

*OUTPATIENT;
    data  cost.&cohrt.&pct._opdcost;
        set  OpEx.&cohrt.&pct._opclms;
        by diag_id;
        where from_dt>=dgn_date ;
           diag_year=year(dgn_date);

            claimStart=from_dt;
            claimEnd=thru_dt;

     *drop procedure occuring before index events;
            %OutPatientCosts;
             retain
                %do i=1 %to &dNum.;
                    op_d%scan(&days.,&i.)
                %end;
            0;

            *Drop first set of variables so we can sum acroos diag_id;
                  if  first.diag_id then do;
                %do i=1 %to &dNum.;
                    op_d%scan(&days.,&i.)=0;
                    %end;
                    end;

                %CreateCostVariables(days=&days.,dNum=&dNum.,pop=op);

                outpat=1;
             keep outpat dgn_date  bene_id  op_: diag_id;
             if last.diag_id  then output;
    run;

 *OUTPATIENT- REVENUE CENTER- Costs are folded into the other variables;

    %mend;


**********************************************************************************;
***    Carrier process Macro     ;
**********************************************************************************;
%macro CarProcess(cohrt=,pct=,days=,dNum=);

    proc sort data=carEx.&cohrt.&pct._carlnits;
        by diag_id;
run;

    proc freq data=carEx.&cohrt.&pct._carlnits;
        table  pmtdnlcd pmtindcd;
    run;

    data  cost.&cohrt.&pct._carcost;
        set carEx.&cohrt.&pct._carlnits;
        *drop procedure occuring before index events;
        where  expnsdt1>=dgn_date and trim(pmtdnlcd) ne "0" ;
        by diag_id;
        length stdPlace $7;
        diag_year=year(dgn_date);
        **************************************************************************;
        ***    Overview of costs;
        **************************************************************************;

        * Total amount paid for medicare services are made up of three parts;
        *  1) Payment made by medicare  ;
        *       a) equal to linepmt;
        * ;
        *  2) PAyment responsibility of the beneficiary;
        *       (variables not currently harmonized) ;
        *       SUM OF{;
        *          a) Line beneficiary part B deductable amount (variable not currently harmonized) ;
        *          b) Line coinsurance amount;
        *              };
        * ;
        *  3) Payment made by Primary Payer ;
        *       (variables not currently harmonized) ;
        *        a) Line beneficiary primary payer amount;
        * ;
        * Summing these 3 parts together gives a measure of total expenditures for health care.;
        * However it is also possible to simply use LALOWCHG (as we do below) which account for;
        * all 3 parts of carrier payments. ;

        * To reiterate:;
        *  (1) Individauls who are interested in the amount medicare reimburses only should use linepmt;
        *  (2) If you are interested in the total amount paid for the services use lalowchg;
        *  (3) If you are interested in a finer definition of costs you may need to add additional;
        *      variables to the harmonization dataset (see excel dictionaries for Carrier files which give a crosswalk.;

        * Lastly note that resdac suggested we remove denied claims/line items which we now do below.;
        * It is worth noting that the variable for denied claims only stretches back to 1997. ;
        * It maybe that prior to 1997, the variable is burried in some other edit code. ;
        * Need to look back at a future point. ;

        * It is equal to zero if the claim is denied.;
        *trim(pmtdnlcd) ne 0
        *not well defined do not use -- if pmtindcd ne 0;

        *Need to aggregate lineitems to the claim levels prior to averging. This is done in the Createcost macro, sums lineitems to the diag_id level;
        stayCost=lalowchg;
        *stayCost=linepmt;

        claimStart=expnsdt1;
        claimEnd=expnsdt2;

        stdPlace = "";


          retain
                %do i=1 %to &dNum.;
                    docof_d%scan(&days.,&i.)
                    docop_d%scan(&days.,&i.)
                    dochosp_d%scan(&days.,&i.)
                    car_d%scan(&days.,&i.)
                    other_d%scan(&days.,&i.)
                %end;
            0;



            *Drop first set of variables so we can sum acroos diag_id;
            if first.diag_id then do;
                %do i=1 %to &dNum.;
                    docof_d%scan(&days.,&i.)=0;
                    docop_d%scan(&days.,&i.)=0;
                    dochosp_d%scan(&days.,&i.)=0;
                    car_d%scan(&days.,&i.)=0;
                    other_d%scan(&days.,&i.)=0;
                    %end;
                    end;


        **************************************************************************;
        ***    CAll Sum costs Macro     ;
        **************************************************************************;
        %CreateCostVariables(days=&days.,dNum=&dNum.,pop=car);
        **THESE GROUPING ARE NOT EXACTLY CORRECT --AMBULANCE-LABS-ER-ETC. GROUPED INTO OTHER;
        if (fileyear <= 1990 and  plcsrvc = "1") or (fileyear > 1990 and plcsrvc = "11")
            then do;
            stdPlace="Office";
            %CreateCostVariables(days=&days.,dNum=&dNum.,pop=docof);
            end;
        if (fileyear <= 1990 and  plcsrvc = "5") or (fileyear > 1990 and plcsrvc = "22")
            then do;
            stdPlace="OPHosp";
            %CreateCostVariables(days=&days.,dNum=&dNum.,pop=docop);
            end;

        if (fileyear <= 1990 and  plcsrvc = "3") or (fileyear > 1990 and plcsrvc = "21")
            then do;
            stdPlace="InHosp";
            %CreateCostVariables(days=&days.,dNum=&dNum.,pop=dochosp);
            end;

        if stdPlace="" then do;
            %CreateCostVariables(days=&days.,dNum=&dNum.,pop=other);
            stdPlace="other";
            end;

        keep dgn_date bene_id  car:  ;
        if last.diag_id then output ;
    run;

    %mend;

**********************************************************************************;
***    MedPAR process Macro     ;
**********************************************************************************;
%macro MedparProcess(cohrt=,pct=,days=,dNum=);
    proc sort data=medEx.&cohrt.&pct._medparclms;
        by diag_id;
    run;

    *Note we only look at costs occuring after the index event;
    data  cost.&cohrt.&pct._mparcost;
        set medEx.&cohrt.&pct._medparclms ;
        by diag_id;
            *drop procedure occuring before index events;
        where admsndt>=dgn_date ;

        claimStart=admsndt;
        claimEnd=dschrgdt;

        %medparStayCost;

            *DEfine variable to retian within do loop-any cost groupings-define here;
        retain
                %do i=1 %to &dNum.;
                    hosp_d%scan(&days.,&i.)
                    snf_d%scan(&days.,&i.)
                    medpar_d%scan(&days.,&i.)
                    shosp_d%scan(&days.,&i.)
                    lhosp_d%scan(&days.,&i.)
                    cah_d%scan(&days.,&i.)
                    rehabh_d%scan(&days.,&i.)
                    psychh_d%scan(&days.,&i.)
                    ohosp_d%scan(&days.,&i.)
                %end;
            0;

            *Drop first set of variables so we can sum acroos diag_id;
               if first.diag_id then do;
                %do i=1 %to &dNum.;
                    hosp_d%scan(&days.,&i.)=0;
                    snf_d%scan(&days.,&i.)=0;
                    medpar_d%scan(&days.,&i.)=0;
                    shosp_d%scan(&days.,&i.)=0;
                    lhosp_d%scan(&days.,&i.)=0;
                    cah_d%scan(&days.,&i.)=0;
                    rehabh_d%scan(&days.,&i.)=0;
                    psychh_d%scan(&days.,&i.)=0;
                    ohosp_d%scan(&days.,&i.)=0;
                    %end;
                    end;

                %CreateCostVariables(days=&days.,dNum=&dNum.,pop=medpar);

                ***SHORT AND LONG TERM HOSPITAL;
                if is_snf=0 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=hosp);
                    end;
                ***SNF COST ONLY;
                if is_snf=1 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=snf);
                    end;
                ***LONG TERM HOSPITAL;
                else if is_longhosp=1 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=lhosp);
                    end;
                ***SHORT TERM HOSPITAL;
                else if is_shorthosp=1 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=shosp);
                    end;
                ***Critical Access hospital ;
                else if is_cah=1 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=cah);
                    end;
                ***Psych Hospital ;
                else if is_psychhosp=1 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=psychh);
                    end;
                ***Rehab Hospital ;
                else if is_rehabhosp=1 then do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=rehabh);
                    end;
                ***OTHER HOSPITAL  ;
                else do;
                    %CreateCostVariables(days=&days.,dNum=&dNum.,pop=ohosp);
                    end;


                keep dgn_date bene_id : diag_id ohosp: rehabh: psychh: cah: shosp: hosp: medpar: snf: lhosp:;
                if last.diag_id  then output ;
                dif=ClaimStart-dgn_date;
    run;

/*    proc print data=cost.&cohrt.&pct._mparcost (obs=100);
    run;
*/
    %mend;


%macro mergeAll(cohrt=,pct=,days=&days,dNum=&dNum.);
    *************************************************************************;
    ***    Prep data for merge     ;
    *************************************************************************;
    proc sort data=indxex.&cohrt.&pct._indx;
        by bene_id dgn_date;
    run;
    %if &pct. ne 100 %then %do;
        proc sort data=cost.&cohrt.&pct._carcost;
            by bene_id dgn_date;
        run;
    %end;

    proc sort data=cost.&cohrt.&pct._mparcost;
        by bene_id dgn_date;
    run;


    proc sort data=cost.&cohrt.&pct._opdcost;
        by bene_id dgn_date;
    run;


    *************************************************************************;
    ***    Prep data for merge     ;
    *************************************************************************;
   data cost.&cohrt.&pct._alldcost;
        merge indxex.&cohrt.&pct._indx(keep=bene_id dgn_date )
            %if &pct. ne 100 %then %do;
            cost.&cohrt.&pct._carcost
                %end;
            cost.&cohrt.&pct._mparcost
            cost.&cohrt.&pct._opdcost;
        by bene_id dgn_date;

        ** Uncomment this code if you want to assign a zero to missing values.;
        /*%AssignMissingZero(Proc=hosp,numDay=&dNum.,Days=&days.);
        %AssignMissingZero(Proc=snf,numDay=&dNum.,Days=&days.);
        %AssignMissingZero(Proc=medpar,numDay=&dNum.,Days=&days.);

        %AssignMissingZero(Proc=op,numDay=&dNum.,Days=&days.);

        %AssignMissingZero(Proc=docof,numDay=&dNum.,Days=&days.);
        %AssignMissingZero(Proc=docop,numDay=&dNum.,Days=&days.);
        %AssignMissingZero(Proc=dochosp,numDay=&dNum.,Days=&days.);
        %AssignMissingZero(Proc=car,numDay=&dNum.,Days=&days.);
        %AssignMissingZero(Proc=other,numDay=&dNum.,Days=&days.);
         */
    run;


        proc means data=cost.&cohrt.&pct._alldcost;
        run;

       *************************************************************************;
       ***    Send to stata      ;
       *************************************************************************;
        x "st &costs./&cohrt.&pct._alldcost.sas7bdat  &costs./&cohrt.&pct._alldcost.dta -y -od";


     proc datasets lib=cost nolist;
           DELETE &cohrt.&pct._carcost
                  &cohrt.&pct._mparcost
                  &cohrt.&pct._opdcost;
    run;

    %mend;



%macro AssignMissingZero(Proc=,numDay=,Days=);
    %do i=1 %to &numDay.;
      if &Proc._d%scan(&days.,&i.)=. then &Proc._d%scan(&days.,&i.)=0;
      %end;
%mend;

**********************************************************************************;
****             START WRAPPER MACRO  ;
**********************************************************************************;

**********************************************************************************;
*WRAPPER MACRO DOCUMENTATION --SAME AS DOC FOUND AT BEGINING OF SCRIPT ---;
**********************************************************************************;
*** It has the following options;
*** cohrt1=    --These reference the ICD-9-CM codes in the code and define what these files will be called.;
*** cohrt2=    -- You can add as many of these as you would like, however if you start running several;
***             -- (4 plus)the efficiently of the program maybe impacted. ;
*** .....;
*** cohrtN=2   -- Define the total number of cohorts you are creating, must equal the number of the last cohorts defined  above;
***               Tells the program how many cohortX to loop through.;
*** pct=05    -- Define the percent extract you are interested in. Use .01% for debugging where possible;

*** win=1      -- Define the  window around the index event for the program to extract.;
***            -- In years.;
**********************************************************************************;


%macro RunProgram(cohrt1=,cohrt2=,cohrt3=,cohrt4=,cohrt5=,cohrtN=,pct=,days=1 3 7 9 15 30 60 90 180 365,dNum=);
    %do j=1 %to &cohrtN.;
        %OutPatProcess(cohrt=&&cohrt&j..,pct=&pct.,days=&days,dNum=&dNum.);
        %MedparProcess(cohrt=&&cohrt&j..,pct=&pct.,days=&days,dNum=&dNum.);
        %if &pct.<100 %then %do;
        %CarProcess(cohrt=&&cohrt&j..,pct=&pct.,days=&days,dNum=&dNum.);
        %end;
        %mergeAll(cohrt=&&cohrt&j..,pct=&pct.,days=&days,dNum=&dNum.);

        %end;
%mend;




%RunProgram(cohrt1=&cohrt.,cohrtN=1,pct=&pct.,days=&day1. &day2. &day3. &day4. &day5. &day6. &day7.,dNum=&dnum.);


/*
********************************************************************************
*** To run this file only or for testing comment out the above lines and;
*** make sure you fill in the correct parameters below;
********************************************************************************;
*%RunProgram(cohrt1=ami,cohrtN=1,pct=05,days=30 90 365,dNum=3);



