** Code which imports Physician RVUs;
** THIS VERSION: Matt Fiedler.
** EARLIER VERSIONS: Maurice Dalton and Nivedhitha Subramanian
**
** Input files for 1995-1998 and 2000-2002 are obtained from the GPO Federal Register
** website and are in the public domain.
**
** The input file for 1999 is obtained from http://www.hcfa.gov/stats/rvu99_c.exe, as archived
** by archive.org.  The CPT information therein is under AMA's copyright and subject to a
** license agreement that prohibits redistribution of the information therein   Thank you to
** Josh Gottlieb for suggesting this way of obtaining this file.
**
** Input files for 2003-present are obtained from the Physician Fee Schedule section of the
** CMS website.  The CPT information therein is under an AMA copyright and, therefore,
** subject to a license agreement that forbids redistribution.

clear all
set more off

///READ IN DIRECTORIES
do ../../../dirInit.do

*** PROCESS 1995-1998 2000-2002 ***
** The regulations for all of these years are available in text format from the GPO website.
** They can be processed in reasonably straightforward manner using a simple algorithm.

//set lines of each year's FR file that should be read
local 1995_lines 6156/16364
local 1996_lines 8225/19147
local 1997_lines 7766/24895
local 1998_lines 6631/23127
local 2000_lines 8072/27384
local 2001_lines 8186/30382
local 2002_lines 10822/31380

foreach y in 1995 1996 1997 1998 2000 2001 2002 {

	// LOAD FILE
	// Note: 1997 and 2002 have some garbage characters that need to be filtered out before use
	if `y' == 1997 {
		tempfile myTmp
		filefilter "$PFSdir/TextFR/CY`y'-FinalRule.htm" `myTmp', from("<INF>") to("") replace
		local curFile `myTmp'
		}
	else if `y' == 2002 {
		tempfile myTmp1
		tempfile myTmp2
		filefilter "$PFSdir/TextFR/CY`y'-FinalRule.htm" `myTmp1', from("<INF>") to("") replace
		filefilter `myTmp1' `myTmp2', from("</INF>") to("")
		local curFile `myTmp2'
		}

	else {
		local curFile "$PFSdir/TextFR/CY`y'-FinalRule.htm"
		}
        di "TEST::$PFSdir"
	infile using $PFSdir/Dictionaries/`y'.dct in ``y'_lines', using(`curFile') clear
	gen year = `y'

	// HANDLE GARBAGE LINES
	// All years except 1995 have blank lines around page breaks and/or descriptions that fall on multiple lines
	// The first line of code handles the page break indicators.  The second handles  blank lines
	// around the page breaks.  The final lines handle the multi-line desriptions.
	if `y' != 1995 {
		drop if hcpcs_cd == "[[Pag" | hcpcs_cd == "age 6" | hcpcs_cd == "age 5"
		drop if hcpcs_cd == "" & description == ""
		replace description = description + " " + description[_n+1] if hcpcs_cd[_n+1] == ""
		drop if hcpcs_cd == ""
		}

	// CLEAN UP OTHER FIELDS

	//simple clean up
	replace modifier = "" if modifier == ".."
	replace description = regexr(description, "\.+$", "")

	//detect footnotes indicating that this code is not used for payment
	if `y' <= 1997 {
		gen not_used_pmt = ""
		replace not_used_pmt = "+" if regexm(work_rvu, "#")
		replace work_rvu = regexr(work_rvu, "#", "")
		}
	else {
		gen not_used_pmt = ""
		replace not_used_pmt = "+" if regexm(work_rvu, "\+")
		replace work_rvu = regexr(work_rvu, "\+", "")
		}

	//deal with odd cases where table contains a letter O instead of a zero
	replace work_rvu = regexr(work_rvu, "O", "0")

	//create NA indicator as in later years' files for facility/non-facility RVUs
	if `y' >= 2000 {
		if `y' == 2002 {
			local statList fi
			}
		else {
			local statList fi trans
			}

		foreach statType of local statList {
			foreach facType in fac nonfac {
				gen `statType'_`facType'_na_ind = ""
				replace `statType'_`facType'_na_ind = "NA" if regexm(`statType'_`facType'_pe_rvu,"NA")
				}
			}
		}

	//destring all the RVUs (getting rid of other flags and NAs in the process)
	foreach rvuVar of varlist *_rvu {
		destring `rvuVar', replace ignore("*#NA")
		confirm numeric variable `rvuVar'
		}

	// Create the in-effect facility/non-facility RVU fields for each year.
	// This facilitates knowing which of the many fields to use for each year.
	if `y' <= 1997 {
		gen ineffect_total_rvu = total_rvu
		label variable ineffect_total_rvu "NBER GENERATED: RVU value actually used for this year"
		}
	else if `y' == 1998 {
		gen ineffect_fac_total_rvu = fac_total_rvu
		label variable ineffect_fac_total_rvu "NBER GENERATED: Facility RVU value actually used for this year"
		gen ineffect_nonfac_total_rvu = nonfac_total_rvu
		label variable ineffect_nonfac_total_rvu ///
		  "NBER GENERATED: Non-facility RVU value actually used for this year"
		}
	else if `y' == 2000 | `y' == 2001 {
		gen ineffect_fac_total_rvu = trans_fac_total_rvu
		label variable ineffect_fac_total_rvu "NBER GENERATED: Facility RVU value actually used for this year"

		gen ineffect_nonfac_total_rvu = trans_nonfac_total_rvu
		label variable ineffect_nonfac_total_rvu ///
		  "NBER GENERATED: Non-facility RVU value actually used for this year"
		}
	else if `y' == 2002 {
		gen ineffect_fac_total_rvu = fi_fac_total_rvu
		label variable ineffect_fac_total_rvu "NBER GENERATED: Facility RVU value actually used for this year"

		gen ineffect_nonfac_total_rvu = fi_nonfac_total_rvu
		label variable ineffect_nonfac_total_rvu ///
		  "NBER GENERATED: Non-facility RVU value actually used for this year"
		}
	saveold "$carRVU/rvu`y'.dta", replace

	}



*** PROCESS 1999, 2003-2011 ****
** For these years, the files can be downloaded directly from the CMS website.
** In each case, we take the last file for that year, which should incorporate all corrections and new codes.
** The original archive from which each file came is included alongside the decompressed text files.

local file1999 PPRRVU99.TXT
local file2003 pprrvu03.txt
local file2004 pprrvu04.txt
local file2005 PPRRVU05.txt
local file2006 PPRRVU06.txt
local file2007 PPRRVU07.txt
local file2008 PPRRVU08.txt
local file2009 PPRRVU09.txt
local file2010a PPRRVU10_jan_may.txt
local file2010b PPRRVU10_OCT_PCT22_V6.txt
local file2011 PPRRVU11_OCT_082511_v2.txt
local file2012a PPRRVU12.txt
local file2012b PPRRVU12_MARUPDT3.txt

foreach y in 1999 2003 2004 2005 2006 2007 2008 2009 2010 2010b 2011 2012a 2012b {

local ypart `y'
if "`y'" == "2010" {
    local ypart 2010a
}
if "`y'" == "2010b" {
    local y     2010
    local ypart 2010b
}
if "`y'" == "2012a" {
    local y     2012
    local ypart 2012a
}
if "`y'" == "2012b" {
    local y     2012
    local ypart 2012b
}

	infile using $PFSdir/Dictionaries/`y'.dct, using("$PFSdir/CMSWebsite/`file`ypart''") clear
	gen year = `y'

	//ensure that there is only a single entry for each hcpcs_cd/modifier pair
	duplicates tag hcpcs_cd modifier, gen(dupTag)
	assert dupTag == 0
	drop dupTag
	assert hcpcs_cd != ""

	// Create the in-effect facility/non-facility RVU fields for each year.
	// This facilitates knowing which of the many fields to use for each year.
	if `y' == 1999 {
		gen ineffect_fac_total_rvu = trans_fac_total_rvu
		label variable ineffect_fac_total_rvu "NBER GENERATED: Facility RVU value actually used for this year"

		gen ineffect_nonfac_total_rvu = trans_nonfac_total_rvu
		label variable ineffect_nonfac_total_rvu ///
		  "NBER GENERATED: Non-facility RVU value actually used for this year"
		}
	else if `y' >= 2003 & `y' <= 2006 {
		gen ineffect_fac_total_rvu = fi_fac_total_rvu
		label variable ineffect_fac_total_rvu "NBER GENERATED: Facility RVU value actually used for this year"

		gen ineffect_nonfac_total_rvu = fi_nonfac_total_rvu
		label variable ineffect_nonfac_total_rvu ///
		  "NBER GENERATED: Non-facility RVU value actually used for this year"
		}
	else if `y' >= 2007 & `y' <= 2012 {
		gen ineffect_fac_total_rvu = trans_fac_total_rvu
		label variable ineffect_fac_total_rvu "NBER GENERATED: Facility RVU value actually used for this year"

		gen ineffect_nonfac_total_rvu = trans_nonfac_total_rvu
		label variable ineffect_nonfac_total_rvu ///
		  "NBER GENERATED: Non-facility RVU value actually used for this year"
		}

	//save file out
	saveold "$carRVU/rvu`ypart'.dta", replace
	}

        ! cp $PFSdir/../fromJoshGottlieb/1992RVU.dta $carRVU/rvu1992.dta -f
        ! cp $PFSdir/../fromJoshGottlieb/1993RVU.dta $carRVU/rvu1993.dta -f
        ! cp $PFSdir/../fromJoshGottlieb/1994RVU.dta $carRVU/rvu1994.dta -f


cd $carRVU
forvalues y=1992/2009 {
use rvu`y',clear
//GOTLIEB FILES DO NOT HAVE STATUS CODE INDICATOR
if "`y'"=="1992" | "`y'"=="1993" |"`y'"=="1994"{
        di "WORKING ON `y'"
    gen mdfr_cd1=mod
    rename mod modifier
    rename hcpcs hcpcs_cd
    gen year=`y'
    des
    gen fac_total_rvu=totalrvus
    gen nonfac_total_rvu=totalrvus
    keep betos year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu
    }
else if "`y'"=="1995" | "`y'"=="1996" | "`y'"=="1997" {
    di "WORKING ON `y'"
    gen mdfr_cd1=modifier
    gen fac_total_rvu=ineffect_total_rvu
    gen nonfac_total_rvu=ineffect_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description update
    des
}
else {
    di "WORKING ON `y'"
    des
    cap drop fac_total_rvu nonfac_total_rvu
    gen mdfr_cd1=modifier
    rename ineffect_nonfac_total_rvu nonfac_total_rvu
    rename ineffect_fac_total_rvu fac_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description
}

    ///generate for mereging later
    gen jan2may2010=0
    gen jun2dec2010=0
saveold rvu`y',replace

di "START CONVERTING TO SAS"
! st rvu`y'.dta rvu`y'.sas7bdat -Y

}


use rvu2010a,clear
    des
    cap drop fac_total_rvu nonfac_total_rvu
    gen mdfr_cd1=modifier
    rename ineffect_nonfac_total_rvu nonfac_total_rvu
    rename ineffect_fac_total_rvu fac_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description
    gen jan2may2010=1
saveold rvu2010a,replace

di "START CONVERTING TO SAS"
! st rvu2010a.dta rvu2010a.sas7bdat -Y

use rvu2010b,clear
    des
    cap drop fac_total_rvu nonfac_total_rvu
    gen mdfr_cd1=modifier
    rename ineffect_nonfac_total_rvu nonfac_total_rvu
    rename ineffect_fac_total_rvu fac_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description
    gen jun2dec2010=1
saveold rvu2010b,replace

di "START CONVERTING TO SAS"
! st rvu2010b.dta rvu2010b.sas7bdat -Y



use rvu2011,clear
    des
    cap drop fac_total_rvu nonfac_total_rvu
    gen mdfr_cd1=modifier
    rename ineffect_nonfac_total_rvu nonfac_total_rvu
    rename ineffect_fac_total_rvu fac_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description
saveold rvu2011,replace

use rvu2012a,clear
    des
    cap drop fac_total_rvu nonfac_total_rvu
    gen mdfr_cd1=modifier
    rename ineffect_nonfac_total_rvu nonfac_total_rvu
    rename ineffect_fac_total_rvu fac_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description
    gen jan2feb2012=1
saveold rvu2012a,replace

di "START CONVERTING TO SAS"
! st rvu2012a.dta rvu2012a.sas7bdat -Y

use rvu2012b,clear
    des
    cap drop fac_total_rvu nonfac_total_rvu
    gen mdfr_cd1=modifier
    rename ineffect_nonfac_total_rvu nonfac_total_rvu
    rename ineffect_fac_total_rvu fac_total_rvu
    keep year hcpcs_cd modifier mdfr_cd1 nonfac_total_rvu fac_total_rvu status_code description
    gen mar2dec2012=1
saveold rvu2012b,replace

di "START CONVERTING TO SAS"
! st rvu2012b.dta rvu2012b.sas7bdat -Y



use rvu2010b,clear
append using rvu2010a
append using rvu2011
append using rvu2012a
append using rvu2012b

forvalues y=1992/2009 {
append using rvu`y'
saveold rvu_weights,replace
}
! st rvu2010a.dta rvu2010a.sas7bdat -Y
! st rvu2010b.dta rvu2010b.sas7bdat -Y
! st rvu2011.dta rvu2011.sas7bdat -Y

! st rvu_weights.dta rvu_weights.sas7bdat -Y

clear






