***********************************************************************;
** TAG PROCEDURES FOUND IN THE DIFFERENT CLAIMS;
***********************************************************************;
** The program defines all procedure coding regimes for specific procedures of interest. It then             ;
** proceeds to tag all claims files with an associated code. Note that instance in which more than one code  ;
** for a procedure exists for a single index event across claims results in several procedures to be tagged. ;
** Look at the end of the program to call the macro;
***********************************************************************;
** Maurice Dalton,  dalton.at.nber.org;
** Based on code originally written by Matt Fiedler in Stata. All errors are my own.;
***********************************************************************;


***********************************************************************;
* READ IN SAS OPTIONS ****;
***********************************************************************;
*Use these options for debugging;
*options symbolgen mlogic mprint mfile ;
%include "../../sasoptions.sas";

*****PRELIMINARIES************
** ALL DIRECTORIES LIVE HERE -- READS IN DIRECTORIES;
%include "../../dirInit.sas";
%include "&SASmac./read_sysparm.sas";

* define libraries;
* define libraries;
libname medEx  "&medparExt.";
libname indxex "&indxExt.";
libname spcar "&spcar.";
libname carEx   "&carExt.";
libname OpEx   "&OpExt.";
libname procs   "&proc.";


* load file containing various simple macros that make programming easier;
%include "&SASmac./UtilityMacros.sas";




*Define ICD9-CM coding regimes for different diagnosis-these are applied to the OP and MedPAR files;
%macro ICD9CMProc(diag=);
    length procType $15;
    procType="";

    icd_prcdr_cd=compress(icd_prcdr_cd," ");
    icd_prcdr_cd=compbl(icd_prcdr_cd);

    %if "&diag."="ami" or "&diag."="acs"  %then %do;
        pci_all=0;
        stent=0;
        cabg=0;
        cath=0;

       *angioplasty and artherectomy (with and without stent insertion);
        * Define PCI based on ICD9-CM Codes;
        if icd_prcdr_cd = "3601"
            or icd_prcdr_cd = "3602"
            or icd_prcdr_cd = "3605"
            or icd_prcdr_cd = "3609"
            or icd_prcdr_cd = "3600"
            or icd_prcdr_cd="360"
            or icd_prcdr_cd="0066"
            then pci_all=1;


       * Define stent based on ICD9-CM Codes- PCI with stent;
        if icd_prcdr_cd = "3606"
            or icd_prcdr_cd = "3607"
            then stent=1;


       * Define cabg  based on ICD9-CM Codes- ;
        if (substr(icd_prcdr_cd,1,3) = "361")
            then cabg=1;


    **CATH (and CABG);
      ***Code only left (372.2) or left & right together (372.3);
      ***Code CG as well 88.55-88.57;
        if  icd_prcdr_cd = "3722"
            or icd_prcdr_cd = "3723"
            or icd_prcdr_cd= "8855"
            or icd_prcdr_cd = "8856"
            or icd_prcdr_cd = "8857"
            then cath=1;

            *Because a cath often time accompanies a pci or CABG (cabg) procedure we automatically code these together.;
            if cabg=1 or pci_all=1 then cath=1;

        if pci_all=1 then procType="pci_all";
        if stent=1 then procType="stent";
        if cabg=1 then procType="cabg";
        if cath=1 then procType="cath";

        %end;
   %mend;

*Define HCPC coding regimes for different diagnosis-these are applied to the OP rev centers and Carrier Claim files;
%macro HCPSProc(year=,diag=);


      *Removes single white space;
    hcpcs_cd=compress(hcpcs_cd," ");
    *Removes multiple blanks;
    hcpcs_cd=compbl(hcpcs_cd);

    length procType $15;
    procType="";

    %if "&diag."="ami" or "&diag."="acs"  %then %do;

        stent=0;
        pci_all=0;
        pci_st=0;
        cabg=0;
        cath=0;

   *PCI;
    *????;
        *92973- percutaneous transluminal coronary thrombectomy--92974- used with Angioplasty;


    **92982/92984 Angioplasty;
    **92995/92996 Artherectomy;
    **93570: Original PTCA code;
        if     (substr(hcpcs_cd,1,5) = "92980")
            or (substr(hcpcs_cd,1,5) = "92981")
            or (substr(hcpcs_cd,1,5) = "92982")
            or (substr(hcpcs_cd,1,5) = "92984")
            or (substr(hcpcs_cd,1,5) = "92995")
            or (substr(hcpcs_cd,1,5) = "92996")
            or (substr(hcpcs_cd,1,5) = "93570")
            or (substr(hcpcs_cd,1,5) = "C1874")
            or (substr(hcpcs_cd,1,5) = "C1875")
            or (substr(hcpcs_cd,1,5) = "G0290")
            or (substr(hcpcs_cd,1,5) = "G0291")
            then pci_all=1;


   *STENT;
    **92980/92981 Bare metal;
    **G0290/G0291 Drug-eluting;
        if (substr(hcpcs_cd,1,5) = "92980")
            or (substr(hcpcs_cd,1,5) = "92981")
            or (substr(hcpcs_cd,1,5) = "G0290")
            or (substr(hcpcs_cd,1,5) = "G0291")
            then stent=1;

   * CABG;
   **33510-33514, 33516 CABG w/ venous grafts ;
   **33517-33523 CABG w/ arterial and venous grafts (escalating numbers of grafts);
   **33530 CABG, re-operation (Dartmouth uses, but set aside for now);
   **33533-33536 CABG w/ arterial grafts (escalating numbers of grafts);
        if (substr(hcpcs_cd,1,5) = "33510")
            or (substr(hcpcs_cd,1,5) = "33511")
            or (substr(hcpcs_cd,1,5) = "33512")
            or (substr(hcpcs_cd,1,5) = "33513")
            or (substr(hcpcs_cd,1,5) = "33514")
            or (substr(hcpcs_cd,1,5) = "33516")
            or (substr(hcpcs_cd,1,5) = "33517")
            or (substr(hcpcs_cd,1,5) = "33518")
            or (substr(hcpcs_cd,1,5) = "33519")
            /*or (substr(hcpcs_cd,1,5) = "33520") Just for checking purposes*/
            or (substr(hcpcs_cd,1,5) = "33521")
            or (substr(hcpcs_cd,1,5) = "33522")
            or (substr(hcpcs_cd,1,5) = "33523")
            /*or (substr(hcpcs_cd,1,5) = "33530") Just for checking purposes*/
            or (substr(hcpcs_cd,1,5) = "33533")
            or (substr(hcpcs_cd,1,5) = "33534")
            or (substr(hcpcs_cd,1,5) = "33535")
            or (substr(hcpcs_cd,1,5) = "33536")
            then cabg=1;

       **CATH (including CABG);
       **For now, coding ALL left and ALL left and right together;
       *93508 Coronary and related cath;
       *93510/11/14 Left, various types;
       *93524,26-29 Combined, various types;
       *93531-95333 Combined, congenital defects;
       *93545: Injection of dye for coronary angiography;
       *93546-93552 Combined cath and cabg (obsolete codes);

        if (substr(hcpcs_cd,1,5) = "93508")
            or (substr(hcpcs_cd,1,5) = "93510")
            or (substr(hcpcs_cd,1,5) = "93511")
            or (substr(hcpcs_cd,1,5) = "93514")
            or (substr(hcpcs_cd,1,5) = "93524")
            or (substr(hcpcs_cd,1,5) = "93526")
            or (substr(hcpcs_cd,1,5) = "93527")
            or (substr(hcpcs_cd,1,5) = "93528")
            or (substr(hcpcs_cd,1,5) = "93529")
            or (substr(hcpcs_cd,1,5) = "93531")
            or (substr(hcpcs_cd,1,5) = "93532")
            or (substr(hcpcs_cd,1,5) = "93533")
            or (substr(hcpcs_cd,1,5) = "93545")
            or (substr(hcpcs_cd,1,5) = "93546")
            or (substr(hcpcs_cd,1,5) = "93547")
            or (substr(hcpcs_cd,1,5) = "93548")
            or (substr(hcpcs_cd,1,5) = "93549")
            or (substr(hcpcs_cd,1,5) = "93550")
            or (substr(hcpcs_cd,1,5) = "93551")
            or (substr(hcpcs_cd,1,5) = "93552")
            then cath=1;


        if pci_all=1 then procType="pci_all";
        if stent=1 then procType="stent";
        if cabg=1 then procType="cabg";
        if cath=1 then procType="cath";

        %end;
    %mend;



%macro OutPatProcess(year=,cohrt=,pct=,sampleRule=,id=);
*The outpatient claims files need to be reshaped because they list several procedure per claim. Hence the first part of this macro ;
*goes through and basically creates a new dataset which tags any of the procedure we are interested in from a particular claim. These files can later;
*be used to identify which claims have a procedure of interests;


***********************************************;
*************OP-CLMS***************************;
***********************************************;


    data ForReshape;
        set OpEx.&cohrt.&pct._opclms ( keep=diag_id dgn_date bene_id fileyear  from_dt thru_dt  prcdr_dt: icd_prcdr_cd:  ) ;
        *drop procedure occuring before index events;
        if from_dt < dgn_date then delete;
        retain ExtClmIndx 0;
        ExtClmIndx= ExtClmIndx+1;
        dgn_year=year(dgn_date);
    run;

    proc contents data=ForReshape;
    run;


    proc sort data=ForReshape;
        by ExtClmIndx diag_id from_dt thru_dt  dgn_year bene_id dgn_date fileyear;
    run;

    *Need to transpose in two steps here because we want to take the code and date procedures from wide to long;
    proc transpose data=ForReshape (keep=ExtClmIndx diag_id dgn_date dgn_year  bene_id fileyear  from_dt thru_dt  icd_prcdr_cd: ) out=OpTempCD prefix=icd_prcdr_cd ;
        by ExtClmIndx diag_id from_dt thru_dt  dgn_year bene_id dgn_date fileyear;
        var icd_prcdr_cd:;
    run;


    proc contents data=OpTempCD;
    run;

    proc transpose data=ForReshape (keep= prcdr_dt:  ExtClmIndx ) out=OpTempDT prefix=prcdr_dt ;
        by ExtClmIndx ;
        var prcdr_dt:;
    run;

    proc contents data=OpTempCD;
    run;

    *Create a new index here which identifies which procedure variables the specific claims came from, need this index to merge below;
    data OpTempCD;
        set OpTempCD;
        procIndx=substr(_name_, 8);
        rename icd_prcdr_cd1=icd_prcdr_cd;
    run;


    data OpTempDT;
        set OpTempDT;
        procIndx=substr(_name_, 8);
        rename prcdr_dt1=prcdr_dt;
    run;

    *Sort for merge -make sure there is no dupes (there shouldnt be);
    proc sort data=OpTempDT out=OpTempDTnodup nodupkey;
        by  ExtClmIndx procIndx;
    run;

    proc sort data=OpTempCD out=OpTempCDnodup nodupkey;
        by  ExtClmIndx procIndx;
    run;


    *Combine Long dates and procedure codes and process the data;
    data procs.&cohrt.&pct._opcPull;
        merge OpTempCDnodup (in=OpCD drop=_name_ _label_) OpTempDTnodup (in=OpDt drop=_name_ _label_) ;
        by ExtClmIndx procIndx;
             **Set harmonized date for CLAIMS only;
        if icd_prcdr_cd="" then delete;
        inCD=OpCD;
        inDT=OpDt;
        if from_dt<dgn_date then delete;
        *Calls macro defined above to tag procedures based on ICD9 proc codes;
        %ICD9CMProc(diag=&cohrt.);

            *Drop claims with procedures not identified -- to begin with its useful to look at all of the claims;
        if procType="" then delete;



        clmStart_opclms=from_dt;
        clmEnd_opclms=thru_dt;

        procDate=prcdr_dt;

            *Since we are assuming that admission date are correct and that procedure dates maybe incorrect;
            *We go ahead and assign missing proc date values to the admission date and also proc date which ;
            *fall outside of the admission dates to be equal to the admission date;
        if  prcdr_dt > thru_dt or prcdr_dt < from_dt or prcdr_dt =. then procDate = from_dt;


        flagAssume=0;

                retain procID_opclms 0;
                procID_opclms=procID_opclms+1;

        keep procID_opclms diag_id dgn_date procDate procID_opclms icd_prcdr_cd procType clmStart_: clmEnd_: flagAssume fileyear bene_id ;

        output;
            *Impose conditions on proc in order to duplicate observations *;

        if procType="stent" then do;
            procType="pci_all";
            pci=1;
            flagAssume=1;
            output;

            end;

        if procType="pci_all" then do;
            procType="cath";
            cath=1;
            flagAssume=1;
            output;
            end;
        if procType="cabg" then do;
            procType="cath";
            cath=1;
            flagAssume=1;
            output;
            end;


    run;


    proc datasets lib=work kill nolist;
    run;


*************OP-REVENUE CENTERS**********************;

    proc sort data=OpEx.&cohrt.&pct._oprevcntrs;
        by fileyear diag_id clm_id ;
    run;

    proc sort data=OpEx.&cohrt.&pct._opclms;
        by fileyear diag_id clm_id ;
    run;

  *Simply process the revenue centers because they only have on HCPCS code per observation;
    data procs.&cohrt.&pct._oprevPull;
        merge OpEx.&cohrt.&pct._oprevcntrs  OpEx.&cohrt.&pct._opclms ;
        by fileyear diag_id clm_id ;
        dgn_year=year(dgn_date);
            *ForRevenueCenters only;
        %HCPSProc(diag=&cohrt.);

            *Drop observations for procedures we are not interested in;
        if procType="" then delete;
        if from_dt<dgn_date then delete;
        claimFile="OPREV";

        procDate=rev_dt;


            *Since we are assuming that admission date are correct and that procedure dates maybe incorrect;
            *We go ahead and assign missing proc date values to the admission date and also proc date which ;
            *fall outside of the admission dates to be equal to the admission date;
        if rev_dt=. or rev_dt>thru_dt or rev_dt < from_dt then procDate=from_dt;

        clmStart_oprevcntrs=from_dt;
        clmEnd_oprevcntrs=thru_dt;



        flagAssume=0;

        retain procID_oprevcntrs 0;
        procID_oprevcntrs=procID_oprevcntrs+1;

        keep bene_id  procID_oprevcntrs claimFile diag_id dgn_date procID_oprevcntrs procDate  procType clmStart_: clmEnd_: dgn_year hcpcs_cd &procVars. flagAssume fileyear;

        output;

        *Impose conditions on proc in order to duplicate observations *;
        if procType="stent" then do;
            procType="pci_all";
            pci=1;
            flagAssume=1;
            output;

            end;

        if procType="pci_all" then do;
            procType="cath";
            cath=1;
            output;
            end;

        if procType="cabg" then do;
            procType="cath";
            cath=1;
            output;
            end;

    run;



    proc datasets lib=work kill nolist;
    run;

    %mend;

*******CARRIER****************************************;
%macro CarProcess(year=,cohrt=,pct=,sampleRule=,id=);

    data  procs.&cohrt.&pct._carPull;
        set carEx.&cohrt.&pct._carlnits ;
        if expnsdt1<dgn_date then delete;
        %HCPSProc(diag=&cohrt.);
        if procType="" then delete;

        length stdPlace $8;
        stdPlace = "Other";
        if (fileyear <= 1990 and  plcsrvc = "1") or (fileyear > 1990 and plcsrvc = "11")
            then stdPlace="Office";
        if (fileyear <= 1990 and  plcsrvc = "3") or (fileyear > 1990 and plcsrvc = "21")
            then stdPlace="OPHosp";
        if (fileyear <= 1990 and  plcsrvc = "5") or (fileyear > 1990 and plcsrvc = "22")
            then stdPlace="InHosp";

        claimFile="CAR";


            *We assume that procedure date are correct in the Carrier file- hence we do not edit missing procedures. This partially occurs because of the structure of the;
            * Carrier files- the claim reported is of the procedure reported. hence the date of the claim is the same as the date as the procedures.           ;
        clmStart_car=expnsdt1;
        clmEnd_car=sexpndt2;

        procDate=expnsdt1;


        flagAssume=0;
        retain procID_car 0;
        procID_car=procID_car+1;

        keep bene_id  procID_car diag_id dgn_date  ProcDate  procType claimFile clmStart_car clmEnd_car  hcpcs_cd stdPlace  &procVars. flagAssume fileyear;

        output;
            *Impose conditions on proc in order to duplicate observations *;
        if procType="stent" then do;
            procType="pci_all";
            pci=1;
            flagAssume=1;
            output;

            end;

        if procType="pci_all" then do;
            procType="cath";
            cath=1;
            flagAssume=1;
            output;
            end;
        if procType="cabg" then do;
            procType="cath";
            cath=1;
            output;
            end;

    run;


    proc datasets lib=work kill nolist;
    run;

/*    proc datasets lib=procs;
        DELETE &cohrt.&pct._carPull;
    run;
    */
    %mend;


%macro MedparProcess(cohrt=,DGNMX=,pct=,sampleRule=,id=);

 *************MEDPAR***************************;
    ** Need to reshape the claim files from long to wide because there are several procedures reported per claims, however;
    ** first I need to make an index of every observation in the claims dataset so that it can be reshaped;
    data ForReshape;
        set medEx.&cohrt.&pct._medparclms (keep=diag_id bene_id fileyear dgn_date prcdrdt: prcdrcd: admsndt dschrgdt dgn_year ) ;
        if admsndt < dgn_date then delete;
        retain ExtClmIndx 0;
        ExtClmIndx= ExtClmIndx+1;
    run;


    proc transpose data=ForReshape (keep=dgn_year dgn_date diag_id bene_id  fileyear admsndt dschrgdt prcdrcd:  ExtClmIndx ) out=LongTempCD prefix=prcdrcd ;
        by ExtClmIndx admsndt dschrgdt diag_id bene_id dgn_date dgn_year fileyear;
        var prcdrcd:;
    run;


    proc transpose data=ForReshape (keep=ExtClmIndx prcdrdt: ) out=LongTempDT prefix=prcdrdt ;
        by ExtClmIndx  ;
        var prcdrdt:;
    run;


    data LongTempCD;
        set LongTempCD;
        procIndx=substr(_name_, 8);
        rename prcdrcd1=prcdrcd;
    run;


    data LongTempDT;
        set LongTempDT;
        procIndx=substr(_name_, 8);
        rename prcdrdt1=prcdrdt;
    run;



    proc sort data=LongTempCD out=LongTempCDnodupe nodupkey;
        by  ExtClmIndx procIndx;
    run;

    proc sort data=LongTempDT out=LongTempDTnodupe nodupkey;
        by  ExtClmIndx procIndx;
    run;

    proc means data=LongTempCD ;
    run;

    proc means data=LongTempDT ;
    run;


    data  procs.&cohrt.&pct._mpPull;
        merge LongTempCDnodupe (in=CD drop=_name_ _label_) LongTempDTnodupe (in=DT drop=_name_ _label_) ;
        by ExtClmIndx procIndx;
        if prcdrcd="" then delete;
        inDT=DT;
        inCD=CD;

        if DT and CD;

        %ICD9CMProc(diag=&cohrt.);
        if admsndt < dgn_date then delete;

        if procType="" then delete;


        claimFile="MEDPAR";
        clmStart_medpar=admsndt;
        clmEnd_medpar=dschrgdt;

            *Since we are assuming that admission date are correct and that procedure dates maybe incorrect;
            *We go ahead and assign missing proc date values to the admission date and also proc date which ;
            *fall outside of the admission dates to be equal to the admission date;
        procDate=prcdrdt;
        if prcdr_dt=. or (prcdr_dt>dschrgdt or prcdr_dt<admsndt) then procDate=admsndt;

        retain procID_medpar 0;
        procID_medpar=procID_medpar+1;

        flagAssume=0;
        keep  bene_id  procID_medpar inDT inCD claimFile diag_id bene_id dgn_date clmStart_: clmEnd_:  procDate
              prcdrcd procType  &procVars. flagAssume ;

        LABEL
            ExtClmIndx = "Index created from extract claims - partial key"
            procIndx   = "Index of procedure variable  - partial key";

        output;
            *Impose conditions on proc in order to duplicate observations *;
        if procType="stent" then do;
            procType="pci_all";
            pci_all=1;
            flagAssume=1;
            output;

            end;
        if procType="pci_all" then do;
            procType="cath";
            cath=1;
            flagAssume=1;
            output;
            end;
        if procType="cabg" then do;
            procType="cath";
            cath=1;
            flagAssume=1;
            output;
            end;

    run;

    %mend;



%macro RunProgram(cohrt1=,cohrt2=,cohrt3=,cohrt4=,cohrt5=,cohrtN=,pct=);


    %do j=1 %to &cohrtN.;
        %if "&&cohrt&j.."="ami" or "&&cohrt&j.."="acs" %then %do;
            **Insert all of the tag names used here;
            %let procVars=%STR(pci_all stent cabg cath );
            %end;
        %MedparProcess(cohrt=&&cohrt&j..,pct=&pct.);
        %OutPatProcess(cohrt=&&cohrt&j..,pct=&pct.);
        %CarProcess(cohrt=&&cohrt&j..,pct=&pct.);


        %end;
    %mend;
/****************************************************************************************************************;
**********CALL THE PROGRAM BY DEFINING THE FOLLOWING WRAPPER PROGRAMS******************************************;
** cohrt1     = Defines the first cohort to loop through;
** cohrt2     = Defines the second cohort to loop through;
** ....

** cohrtN     = The number of cohorts defined.
*
** pct        = The percent of interest
************************************************************************************************************r**;
***************************************************************************************************************;
*/

%RunProgram(cohrt1=&cohrt.,cohrtN=1,pct=&pct.);
*%RunProgram(cohrt1=ami,cohrtN=1,pct=20);



/*
*Sample code to write reports, need to fill in macro variables and directory locations.
   *Output procedures  to excel;
ODS HTML FILE="..../Output/Proc/&cohrt.&pct._OPRev_procType.xls" STYLE=statistical;
TITLE "Long Claim File-Procedure Codes by year-HCCS CODES USED IN to define Procedures";
proc freq data=procs.&cohrt.&pct._OpRevProcPull;
    table  procType*dgn_year /norow nocol nopercent ;
run;
ODS HTML  CLOSE;


            *Output procedure codes to excel;
ODS HTML FILE="..../Output/Proc/&cohrt.&pct._all_procType.xls" STYLE=statistical;
TITLE "Combined procedures";
proc freq data=procs.&cohrt.&pct._ProcPull ;
    table  proc*dgn_year procs*claimFile /norow nocol nopercent ;
run;

proc freq data=procs.&cohrt.&pct._ProcPull_&sampleRule.WIDE_&id. ;
    table  sum_: /norow nocol nopercent ;
run;

ODS HTML  CLOSE;


*Output raw procedure codes to excel-;
ODS HTML FILE="..../Output/Proc/&cohrt.&pct._medPar_procType.xls" STYLE=statistical;
TITLE "Procedures  by year";
proc freq data= procs.&cohrt.&pct._medparProcPull;
    table  procType*dgn_year /norow nocol nopercent ;
run;
ODS HTML  CLOSE;


    *Output procedure codes to excel;
ODS HTML FILE="..../Output/Proc/&cohrt.&pct._OPClsum_procType.xls" STYLE=statistical;
TITLE "Long Claim File-Procedure Codes by year";
proc freq data=procs.&cohrt.&pct._OPCProcPull;
    table  procType*dgn_year /norow nocol nopercent ;
run;
ODS HTML  CLOSE;



