***********************************************************************;
** CREATE A FUZZY MERGE BETWEEN TAGGED PROCEDURES FOUND IN THE DIFFERENT CLAIMS;
***********************************************************************;
** This program aims to match procedures codes in the institutional (MedPAR and OP) files and match it with;
** the Carrier files. We want to do this in order to be sure to include all procedures, which are more likely,;
** to be reported in the carrier files and also to use the two files two cross-check the carrier procedures.;
**
** This program is the second part of a two step program (Step1_tagProcs.sas), the first step tags each procedure;
** code which lies within a procedure code grouping, IE CABG. Next this program takes those groupings and attempts;
** to first match Carrier to OP/MedPAR files based on the proctype and group date.;
**
** Next we attempt to identify carrier claims which fall within the institutional admission and discharge dates. This fuzzy-type matching;
** is next followed by identifying procedures which only lie within a single claim file.;
***********************************************************************;
** Maurice Dalton,  dalton.at.nber.org;
** Based on code originally written by Matt Fiedler in Stata. All errors are my own.;
***********************************************************************;


***********************************************************************;
* READ IN SAS OPTIONS ****;
***********************************************************************;
*Use these options for debugging;
*options symbolgen mlogic mprint mfile ;
%include "../../sasoptions.sas";

*****PRELIMINARIES************;
** ALL DIRECTORIES LIVE HERE -- READS IN DIRECTORIES;
%include "../../dirInit.sas";
%include "&SASmac./read_sysparm.sas";
* define libraries;
libname indxex "&indxExt.";
libname carEx   "&carExt.";
libname OpEx   "&OpExt.";
libname procs   "&proc.";

*load file containing various simple macros that make programming easier;
%include "&SASmac./UtilityMacros.sas";


***BEGIN PROCEDURE;
%macro StartProcedure(diag=&diagW.,pct=&pctW.,numDayw=);


*Sort files and output to work directories, keeping original files untouched;
    proc sort data=procs.&diag.&pct._mpPull  out=med ;
        by diag_id procType procDate;
    run;

    proc sort data=procs.&diag.&pct._carPull  out=car ;
        by diag_id procType procDate;
    run;


    proc sort data= procs.&diag.&pct._opcPull    out=OpC ;
        by diag_id procType procDate;
    run;

    proc sort data= procs.&diag.&pct._oprevPull  out=OpR ;
        by diag_id procType procDate;
    run;



**EXACT MATCH: CAR-->MEDPAR;
proc sql;
            create table ExactCar_med as
                select *
                from work.car (keep=diag_id bene_id  procDate procType procID_car ) C , work.med(keep=diag_id procDate procType procID_medpar)  M
                where (C.diag_id = M.diag_id)
                and   (C.procDate = M.ProcDate)
                and   (C.procType = M.procType);
                quit;




**EXACT MATCH: CAR-->OPC;
proc sql;
            create table ExactCar_opc as
                select *
                from work.car (keep=diag_id bene_id  procDate procType procID_car ) C , work.opc(keep=diag_id procDate procType procID_opclms )  M
                where (C.diag_id = M.diag_id)
                and   (C.procDate = M.ProcDate)
                and   (C.procType = M.procType);
                quit;



**EXACT MATCH: CAR-->OPR;
proc sql;
            create table ExactCar_opr as
                select *
                from work.car (keep=diag_id bene_id  procDate procType procID_car ) C , work.opr(keep=diag_id bene_id procDate procType procID_oprevcntrs)  M
                where (C.diag_id = M.diag_id)
                and   (C.procDate = M.ProcDate)
                and   (C.procType = M.procType);
                quit;




*Sort and get rid of duplicates,based on files in the carrier file;
proc sort data=ExactCar_opr out=ExactCar_opr_carID nodupkey;
by procType procID_car;
run;

proc sort data=ExactCar_opc out=ExactCar_opc_carID nodupkey;
by procType procID_car;
run;

proc sort data=ExactCar_med out=ExactCar_med_carID nodupkey;
by procType procID_car;
run;

*Sort for merge;
    proc sort data=car;
by procType procID_car;
run;


*ID ALL CARRIER CLAIMS WITH AN EXACT MATCH AND ALL CLAIMS WITHOUT;
data ExactMerge;
    merge car          (in=c keep=diag_id procDate procType procID_car dgn_date)
          ExactCar_opr_carID (in=opr keep=procID_car procType procID_oprevcntrs)
          ExactCar_opc_carID (in=opc keep=procID_car procType procID_opclms)
          ExactCar_med_carID (in=m keep=procID_car procType procID_medpar);
    by procType procID_car;

    ExactCar_med=0 ;
    if  c=1 and m=1 then ExactCar_med=1;

    ExactCar_opc=0 ;
    if  c=1 and opc=1 then ExactCar_opc=1;

    ExactCar_opr=0 ;
    if  c=1 and opr=1 then ExactCar_opr=1;

    CarOnly=0;
    *Label unmatched carrier files;
    if ExactCar_med=0 and ExactCar_opc=0 and ExactCar_opr=0 and c=1 then CarOnly=1;
        *CarOnly=1;
run;

proc freq data=ExactMerge;
    table ExactCar_med*procType;
run;



**START FUZZ--MATCHES IN WHICH THE CARRIER PROC TYPEs DATE FALLS WITHIN THE INSTITUTIONAL STAY/CLAIM DATE;

**FUZZY MATCH: CAR-->MEDPAR;
proc sql;
            create table FuzzyCar_med as
                select *
                from work.car (keep=diag_id procDate procType procID_car ) C , work.med(keep=diag_id procType clmEnd_medpar clmStart_medpar procID_medpar)  M
                where (C.diag_id = M.diag_id)
                and   (C.procDate >= M.clmStart_medpar and C.procDate <= M.clmEnd_medpar)
                and   (C.procType = M.procType);
                quit;

**FUZZY MATCH: CAR-->OPC;
proc sql;
            create table FuzzyCar_opc as
                select *
                from work.car (keep=diag_id procDate procType procID_car ) C , work.opc(keep=diag_id  clmEnd_opclms clmStart_opclms procType procID_opclms)  M
                where (C.diag_id = M.diag_id)
                and    (C.procDate >= M.clmStart_opclms and C.procDate <= M.clmEnd_opclms)
                and   (C.procType = M.procType);
                quit;

**FUZZY MATCH: CAR-->OPR;
proc sql;
            create table FuzzyCar_opr as
                select *
                from work.car (keep=diag_id procDate procType procID_car ) C , work.opr(keep=diag_id clmEnd_oprevcntrs clmStart_oprevcntrs procType procID_oprevcntrs)  M
                where (C.diag_id = M.diag_id)
                and    (C.procDate >= M.clmStart_oprevcntrs and C.procDate <= M.clmEnd_oprevcntrs)
                and   (C.procType = M.procType);
                quit;


*Since we used proc sql to do a crossproduct type merge, some duplicates could have been create;
*In this step we get rid of duplicates, by the CarrierIDs and later use these ID to merge in the correct files;
proc sort data=FuzzyCar_med out=FuzzyCar_med_carID nodupkey;
by procType procID_car;
run;

proc sort data=FuzzyCar_opc out=FuzzyCar_opc_carID nodupkey;
by procType procID_car;
run;

proc sort data=FuzzyCar_opr out=FuzzyCar_opr_carID nodupkey;
by procType procID_car;
run;




*Merge fuzzy matches to the Carrier master claim;
data FuzzyMerge;
    merge car (in=c keep=diag_id procID_car procType procDate dgn_date)
          FuzzyCar_opr_carID (in=opr keep=procID_car procType  procID_oprevcntrs )
          FuzzyCar_opc_carID (in=opc keep= procID_car procType  procID_opclms)
          FuzzyCar_med_carID (in=m keep= procID_car procType  procID_medpar);
*We merge procID_car because they have been SQLed to keep Car procedure dates.;
    by procType procID_car;

    FuzCar_med=0 ;
    if   m=1 then FuzCar_med=1;

    FuzCar_opc=0 ;
    if   opc=1 then FuzCar_opc=1;

    FuzCar_opr=0 ;
    if   opr=1 then FuzCar_opr=1;


    CarOnly=0;
    *Label unmatched carrier files;
    if FuzCar_med=0 and FuzCar_opc=0 and FuzCar_opr=0 and c=1 then CarOnly=1;
run;



proc sort data=FuzzyMerge;
    by procType procID_car;
run;

proc sort data=ExactMerge;
    by procType procID_car;
run;

*Determine matched carrier files;
data CarMatches;
    merge FuzzyMerge(in=F) ExactMerge(in=E);
    by procType procID_car;
run;



*Next we are interested in identifying the claims which are not in the carrier files;
*Hence we merge in the carrier matches back into the institutional claims. First we need to;
*sort by the correct ID var and get rid of dupes by institutional files this time (rather then by the carrier ID as before);

proc sort data=FuzzyCar_med out=FuzzyCar_med_medID nodupkey;
by procType procID_medpar;
run;

proc sort data=FuzzyCar_opc out=FuzzyCar_opc_opcID nodupkey;
by procType procID_opclms;
run;

proc sort data=FuzzyCar_opr out=FuzzyCar_opr_oprID nodupkey;
by procType procID_oprevcntrs;
run;



proc sort data=ExactCar_opr out=ExactCar_opr_oprID nodupkey;
by procType procID_oprevcntrs;
run;

proc sort data=ExactCar_opc out=ExactCar_opc_opcID nodupkey;
by procType procID_opclms;
run;

proc sort data=ExactCar_med out=ExactCar_med_medID nodupkey;
by procType procID_medpar;
run;


*Create datasets which hold all matched claim indexes in them;
data MedInCar;
    set FuzzyCar_med_medID ExactCar_med_medID  ;
    keep procType procID_medpar ;
run;

    data opcInCar;
    set FuzzyCar_opc_opcID ExactCar_opc_opcID  ;
    keep procType procID_opclms ;
run;

    data oprInCar;
    set FuzzyCar_opr_oprID ExactCar_opr_oprID    ;
    keep procType procID_oprevcntrs ;
run;


*Prepare files for merge;
proc sort data=med ;
    by procType procID_medpar;
run;

proc sort data=opc;
    by procType procID_opclms;
run;

proc sort data=opr;
    by procType  procID_oprevcntrs;
run;


proc sort data=MedInCar nodupkey;
    by procType procID_medpar;
run;

proc sort data=opcInCar nodupkey;
    by procType procID_opclms;
run;

proc sort data=oprInCar nodupkey;
    by procType  procID_oprevcntrs;
run;



*Merge original file and the previously created matched index holder  files and drop obs in the latter;
data HospUnMatched;
    merge med(in=m keep=diag_id procDate procType procID_medpar dgn_date) MedInCar(in=f keep=procType procID_medpar);
    by   procType procID_medpar  ;
    if f=1 then delete;
    medOnly=1;
    inM=m;
    inF=f;
    keep inM inF medOnly diag_id procType procDate  procID_medpar dgn_date;
run;

data OPCUnMatched;
    merge opc(in=m keep=diag_id procDate procType procID_opclms dgn_date) opcInCar(in=f keep=procType procID_opclms);
    by   procType procID_opclms ;
    if f=1 then delete;
    opcOnly=1;
    keep opcOnly diag_id procType procDate  dgn_date procID_opclms ;
run;


data OPRUnMatched;
    merge opr(in=m keep=diag_id procDate procType procID_oprevcntrs dgn_date) oprInCar (in=f keep=procType procID_oprevcntrs) ;
    by   procType procID_oprevcntrs  ;
    if f=1 then delete;
    oprOnly=1;
    keep oprOnly diag_id procType procDate  procID_oprevcntrs dgn_date ;
run;


*Combine all procedures and create indexes;
data AllProcedure ;
set HospUnMatched OPCUnMatched OPRUnMatched CarMatches;

if FuzCar_med=. then FuzCar_med=0;
if FuzCar_opc=. then FuzCar_opc=0;
if FuzCar_opr=. then FuzCar_opr=0;

if ExactCar_opr=. then ExactCar_opr=0;
if ExactCar_opc=. then ExactCar_opc=0;
if ExactCar_med=. then ExactCar_med=0;

if opcOnly=. then opcOnly=0;
if oprOnly=. then oprOnly=0;
if medOnly=. then medOnly=0;
if CarOnly=. then CarOnly=0;

if FuzCar_med=1 or FuzCar_opc=1 or FuzCar_opr=1 or ExactCar_opr=1 or ExactCar_opc=1 or ExactCar_med=1 then CarOnly=0;
if FuzCar_med=1 or FuzCar_opc=1 or FuzCar_opr=1 or ExactCar_opr=1 or ExactCar_opc=1 or ExactCar_med=1 then medOnly=0;
if FuzCar_med=1 or FuzCar_opc=1 or FuzCar_opr=1 or ExactCar_opr=1 or ExactCar_opc=1 or ExactCar_med=1 then opcOnly=0;
if FuzCar_med=1 or FuzCar_opc=1 or FuzCar_opr=1 or ExactCar_opr=1 or ExactCar_opc=1 or ExactCar_med=1 then oprOnly=0;

if opcOnly=. then opcOnly=0;
if oprOnly=. then oprOnly=0;
if medOnly=. then medOnly=0;
if CarOnly=. then CarOnly=0;


if ExactCar_med=1 then medOnly=0 ;
if ExactCar_opc=1 then opcOnly=0 ;
if ExactCar_opr=1 then oprOnly=0 ;

if FuzCar_med=1 then medOnly=0 ;
if FuzCar_opc=1 then opcOnly=0 ;
if FuzCar_opr=1 then oprOnly=0 ;

ProcDif=procDate-dgn_date;
keep ProcDif dgn_date procDate procType diag_id Fuz: Exact: CarOnly medOnly opcOnly oprOnly;

run;




proc freq data=AllProcedure;
    tables Fuz: Exact: CarOnly medOnly opcOnly oprOnly;
run;

proc means data=AllProcedure;
run;


proc sort data=AllProcedure;
    by diag_id;
run;

proc print data=AllProcedure (obs=100);
run;



    data All (keep=diag_id cabg_: pci_all_: stent_: cath_:);
        set AllProcedure ;
        by diag_id;
        retain %do i=1 %to &numDayw.;
            cath_day%scan(&Days.,&i.)
            cabg_day%scan(&Days.,&i.)
            stent_day%scan(&Days.,&i.)
            pci_all_day%scan(&Days.,&i.)
            %end; 0 ;

            if first.diag_id then do;
        %do i=1 %to &numDayw.;
                cath_day%scan(&Days.,&i.)=0;
                cabg_day%scan(&Days.,&i.)=0;
                stent_day%scan(&Days.,&i.)=0;
                pci_all_day%scan(&Days.,&i.)=0;
                %end;
            end;

* ProcDif defined in prior datastep;

            ProcDif=procDate-dgn_date;

            if procType="cath" then do;
                %AssignDays(numDay=&numDayw.,Proc=cath);
                end;
            if procType="cabg" then do;
                %AssignDays(numDay=&numDayw.,Proc=cabg);
                end;
            if procType="pci_all" then do;
                %AssignDays(numDay=&numDayw.,Proc=pci_all);
                end;
            if procType="stent" then do;
                %AssignDays(numDay=&numDayw.,Proc=stent);
                end;

*Do not allow CABG to occur in OP;
            if procType="cabg" then do;
                if opcOnly=1 or oprOnly=1 then delete;
                end;


            if last.diag_id then output All;
    run;


    proc print data=All (obs=10);
        run;


proc means data=All ;
run;


*Map back to index events;
*create a measure here of procedure by date;

proc sort data=indxex.&diag.&pct._indx;
    by diag_id;
run;

data procs.&diag.&pct._procedures;
    merge indxex.&diag.&pct._indx (in=long keep=diag_id bene_id dgn_date) All ;
    by diag_id;
    %AssignMissingZero(Proc=cath,numDay=9);
    %AssignMissingZero(Proc=cabg,numDay=9);
    %AssignMissingZero(Proc=stent,numDay=9);
    %AssignMissingZero(Proc=pci_all,numDay=9);
    keep diag_id  cabg_: pci_all_: cath_: stent_: bene_id dgn_date;
    if last.diag_id then output;
run;




     x "st &proc./&diag.&pct._procedures.sas7bdat  &proc./&diag.&pct._procedures.dta -y -od";
%mend;


Proc datasets lib=work kill;
run;



***THIS MACRO IS CALLED ABOVE***;
%macro AssignDays(Proc=,numDay=&numDayw.);
*this macro takes the variables which have been merged together and they;
*Assign 0 where the value was missing before;
    %do i=1 %to &numDay.;
      &Proc._day%scan(&Days.,&i.)=0;
      if ProcDif<=%scan(&Days.,&i.) and procType="&Proc."  then &Proc._day%scan(&Days.,&i.)= 1 + &Proc._day%scan(&Days.,&i.);
      %end;
%mend;
%Let Days =1 3 7 15 30 60 90 180 365;
%macro AssignMissingZero(Proc=,numDay=9);
    %do i=1 %to &numDay.;
      if &Proc._day%scan(&Days.,&i.)=. then &Proc._day%scan(&Days.,&i.)=0;
      %end;

    %mend;
***THIS MACRO IS CALLED ABOVE***;


***************************************************************************************************************;
**********CALL THE MACRO  BY DEFINING THE FOLLOWING PARAMETERS******************************************;
***************************************************************************************************************;

** diag       = Defines the cohort of interest. Unlike other programs only one cohort can be called at a time in this sas program;
** ;
** pct        = The percent of interest;

** numDayw    = is a parameter which is equal to the number of different days included in the measure.;
**              It basically tells the program when to stop the loop, t the 9th day entry.;
**
***************************************************************************************************************;
***************************************************************************************************************;

*Define the number of day increments interested in measuring in the below macro;
%Let Days =1 3 7 15 30 90 180 365;
%StartProcedure(diag=&cohrt.,pct=&pct.,numDayw=8);
*%StartProcedure(diag=ami,pct=20,numDayw=8);





