*/*******************************************************************************************
*  Cost summary for the  Medpar-Outpatient/SNF-
* By: Maurice Dalton, daltonm@nber.org, March 24, 2010
* Based on modified code provided by Matt Fiedler
*
* The purpose of this do file is to create some simple cost measures from the medpar files.
* Costs will be broken down by Medicare A,B, and C as well as tryinh to make sense of over time.

*********************************************************************************************/

/* PRELIMINARIES */
set more off
clear
pause off
set memory 1000M
// REad in directories from this file
do ../../dirInit.do

global statexw "/disk/agedisk1/medicare.work/wise-DUA16702/daltonm/summary"
global longExt "/disk/agero5/medicare.work/wise-DUA16702/daltonm/LongExtract"
global medparCost "/homes/nber/daltonm/Medicare/Output/InSNFCost"

local pct 05

///Define the total number of diagnosis variables used to index events (principle versus any secondary diagnosis)
local DGNMX 1

local diag="ami"
//preliminaries
use "$longExt/`diag'`pct'_`DGNMX'_claims_mn.dta"   , clear

//look only at post-AMI admissions

 keep if sadmsndt >= dgn_date


//DEFINE COST VARIABLES
//The goal is to obtain an amount equal to the base DRG payment plus
//any outlier payments.  This exlcludes:
// 1) Indirect medical education payments;
// 2) Disproportionate share payments;
// 3) Pass-through costs (capital before 1991, direct medical education,
//    organ acquisition costs, and certain other costs)
// 4) Prospective payment capital payments;
//
//For certain purposes, it may be desirable to include these amounts.  Note,
//however, that for the latter two items, this is not possible for
//the full 1984-2005 period.
//
//In particular, consistent and accurate pass-through fields are not available
//for 1984-1986. Furthermore, capital costs were paid through prospective
//payment for 1991-2005, but on a pass-through basis prior to 1991.  This raises
//conceptual difficulties, plus the lack of good pass-through fields for all years
//means that it is not possible to generate a consistent series that includes
//these costs.
//
//Toward this end, the code starts with the drgprice, which is defined as described
//at http://www.resdac.umn.edu/ddme/NewFiles/DRGPRICE.htm and
//
//ADDS outlier payments since we want to include these
//
//SUBTRACTS ime_amt for discharges after 10/1/1988 since they are added into
//the Medicare Payment Amount variable and, thus, appear in DRGPRICE
//(see http://www.resdac.umn.edu/ddme/NewFiles/PMT_AMT.htm)
//
//SUBTRACTS capital payments by subtracting off pps_cptl
//(note that field is missing for 1984-1986, but there are no payments for
//these years)
//
//SUBTRACTS disproportionate share payments by substracting off disp_shr
//(note that field is missing for 1984-1986, but there are no payments for
//these years)
//
//Pass through payments are not included in DRGPRICE and, so, do not require
//special handling here

gen stayCost = drgprice + outlramt
replace stayCost = stayCost - ime_amt if sdschrgdt >= mdy(10,1,1988)
replace stayCost = stayCost - pps_cptl if pps_cptl != .
replace stayCost = stayCost - disp_shr if disp_shr != .



// Create separate cost variables for SNF and hospital dtays
gen stayCost_snf = 0
replace stayCost_snf = stayCost if is_snf == 1

gen stayCost_In=stayCost
replace stayCost_In = 0 if is_snf == 1

/// If your intrested in regions
/// merge prvstate using "$statexw/MedicareStateCrosswalk.dta", sort uniqusing nokeep
///  table diag_year region if region != "O", c(mean stayCost)


/* SPENDING BREAKDOWNS BY TIME AFTER AMI */



  gen admit_t1  = (sadmsndt - dgn_date) <= 1
  gen admit_t3 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 3)
  gen admit_t7 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 7)
  gen admit_t9 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 9)
  gen admit_t15 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 15)
  gen admit_t30 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 30)
  gen admit_t90 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 90)
  gen admit_t180 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 180)
  gen admit_t365 = ((sadmsndt - dgn_date) >= 0) & ((sadmsndt - dgn_date) <= 365)



  gen admitcompile=1 if admit_t1==1
  replace admitcompile=2 if admit_t3==1
  replace admitcompile=3 if admit_t7==1
  replace admitcompile=4 if admit_t9==1
  replace admitcompile=5 if admit_t15==1
  replace admitcompile=6 if admit_t30==1
  replace admitcompile=7 if admit_t90==1
  replace admitcompile=8 if admit_t180==1
  replace admitcompile=9 if admit_t365==1



///Collapse across diagnosis encounters
/*
foreach i in 1 3 7 9 15 30 90 180 365 {
preserve
  collapse (sum) stayCost (first) diag_year  , by(is_snf diag_id admit_t`i')
  drop admit_t`i'
  gen admit_t`i'=1
  save data`i',replace
restore
}


use data1,clear
append using  data3 data7 data9 data15 data30 data90 data180 data365
*/

///Create summary statistics by date
foreach i in 1 3 7 9 15 30 90 180 365 {
  statsmat stayCost if is_snf ==1 & admit_t`i'==1 ,by(diag_year)  s(mean) mat(cost`i'SNF)  f(%4.0f)



  statsmat stayCost if is_snf ==0 & admit_t`i'==1 ,by(diag_year)  s(mean) mat(cost`i'IN)  f(%4.0f)
  statsmat stayCost if admit_t`i'==1            ,by(diag_year)  s(mean) mat(cost`i')  f(%4.0f)


  if `i'== 1 {
    local days "less then 1 days"
  }
  if `i'== 3 {
    local days "less then 3 days"
  }
  if `i'== 7 {
    local days "less then 7 days"
  }
  if `i'== 9 {
    local days "less then 9 days"
  }
  if `i'== 15 {
    local days "less then 15 days"
  }
  if `i'== 30 {
    local days "less then 30 days"
  }
  if `i'== 90 {
    local days "less then 90 days"
  }
  if `i'== 180 {
    local days "less then 180 days"
  }
  if `i'==365  {
    local days "less then 365 days"
  }


  mat colnames cost`i'SNF = "`days'"
  mat colnames cost`i'IN  = "`days'"
  mat colnames cost`i'    = "`days'"

  local costSNF=cost`i'SNF, costSNF
  local costIN=cost`i'SNF, costIN
   local cost=cost`i', cost
  }

statsmat stayCost if is_snf ==1  ,by(diag_year)  s(mean) mat(costSNF)  f(%4.0f)
statsmat stayCost if is_snf ==0  ,by(diag_year)  s(mean) mat(costIN)  f(%4.0f)
statsmat stayCost                ,by(diag_year)  s(mean) mat(costT)  f(%4.0f)

  mat colnames costSNF = Total
  mat colnames costIN  = Total
  mat colnames costT    = Total



 mat CostSNF=(`costSNF' costSNF)
 mat CostIN=(`costIN' costIN)
 mat CostAll  =(`cost' costT)

 mat rownames CostSNF=SNF:
 mat rownames CostIN =Inpatient:
 mat rownames CostAll   =Total:

 mat cost=(CostSNF \ CostIN \ CostAll)


///NEED THIS IF STATEMENT TO MAKE THE FIRST ITERATION REPLACE XML FILE
/*if "`diag'"=="chf" {
xml_tab cost using "$medparCost/CostInSNF.xml",  sheet(InSNF_`diag',nogridlines)  format((S3110) (S3230 N3120))  lines(COL_NAMES 4 LAST_ROW 13 SCOL_NAMES 2)   cblanks(7 14) tblanks(3) font("Garamond" 11)  title("Mean Diagnosis Encounter costs at Inpatient/SNF facilitiy")  note("The amounts reported here are equal to the DRG payment plus any outlier payments.  This excludes:   1) Indirect medical education payments; 2) Disproportionate share payments;  3) Pass-through costs (capital before 1991, direct medical education,  organ acquisition costs, and certain other costs)  4) Prospective payment capital payments;") replace
}
if "`diag'"!="chf" {
xml_tab cost  using "$medparCost/CostInSNF.xml",  sheet(InSNF_`diag',nogridlines)   format((S3110) (S3230 N3200))  lines(COL_NAMES 4 LAST_ROW 13 SCOL_NAMES 2)     cblanks(7 14) tblanks(3) font("Garamond" 11)  title("Mean Diagnosis Encounter costs at Inpatient/SNF facilitiy")  note("The amounts reported here are equal to the DRG payment plus any outlier payments.  This excludes:    1) Indirect medical education payments;   2) Disproportionate share payments;    3) Pass-through costs (capital before 1991, direct medical education, organ acquisition costs, and certain other costs) 4) Prospective payment capital payments;") append
}*/


/// Calculate for means
foreach i in 1 3 7 9 15 30 90 180 365 {
  statsmat stayCost if is_snf ==1 & admit_t`i'==1  ,by(diag_year)  s(mean) mat(cost`i'SNF)  f(%4.0f)
  statsmat stayCost if is_snf ==0 & admit_t`i'==1  ,by(diag_year)  s(mean) mat(cost`i'IN)  f(%4.0f)
  statsmat stayCost if admit_t`i'==1               ,by(diag_year)  s(mean) mat(cost`i')  f(%4.0f)


    if `i'== 1 {
    local days "less then 1 days"
  }
  if `i'== 3 {
    local days "less then 3 days"
  }
  if `i'== 7 {
    local days "less then 7 days"
  }
  if `i'== 9 {
    local days "less then 9 days"
  }
  if `i'== 15 {
    local days "less then 15 days"
  }
  if `i'== 30 {
    local days "less then 30 days"
  }
  if `i'== 90 {
    local days "less then 90 days"
  }
  if `i'== 180 {
    local days "less then 180 days"
  }
  if `i'==365  {
    local days "less then 365 days"
  }


  mat colnames cost`i'SNF= "`days'"
  mat colnames cost`i'IN= "`days'"
  mat colnames cost`i'= "`days'"



}

  statsmat stayCost if is_snf ==1  ,by(diag_year)  s(mean) mat(costSNF)  f(%4.0f)
  statsmat stayCost if is_snf ==0  ,by(diag_year)  s(mean) mat(costIN)  f(%4.0f)
  statsmat stayCost                ,by(diag_year)  s(mean) mat(costT)  f(%4.0f)

  mat colnames costSNF = Total
  mat colnames costIN  = Total
  mat colnames costT   = Total

mat CostSNF=(`costSNF' costSNF)
mat CostIN=(`costIN' costIN)
mat CostALL  =(`cost' costT)

  mat rownames CostSNF=SNF:
  mat rownames CostIN =Inpatient:
  mat rownames CostALL=Total:

mat cost=(CostSNF \ CostIN \ CostALL)

! rm $medparCost/CostInSNF.xml -f
xml_tab cost using"$medparCost/CostInSNF.xml",  sheet(InSNF_`diag'_Mean,nogridlines)   format((S3110) (S3230 N3200))  lines(COL_NAMES 4 LAST_ROW 13 SCOL_NAMES 2)     cblanks(7 14) tblanks(3) font("Garamond" 11)  title("Mean Diagnosis Encounter costs at Inpatient/SNF facilitiy")  note("The amounts reported here are equal to the DRG payment plus any outlier payments.  This excludes:    1) Indirect medical education payments;    2) Disproportionate share payments;    3) Pass-through costs (capital before 1991, direct medical education,       organ acquisition costs, and certain other costs)    4) Prospective payment capital payments;")


}

